<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromCollection;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\Exportable;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithStyles;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;

class ObserverDetailReportsExport implements FromCollection, WithHeadings, ShouldAutoSize, WithStyles
{
    use Exportable;

    protected $data;

    public function __construct(Collection $data)
    {
        $this->data = $data;
    }

    public function collection()
    {
        return $this->data;
    }

    public function headings(): array
    {
        return [
            'NTA ID',
            'City Coordinators',
            'Observers/Dy.Observers',
            'Contact No.',
            'PAN NO.',
            'Duty Days',
            'Mock',
            'Remuneration',
            'Mock Remuneration',
            'Local Conveyance',
            'Food/-',
            'Scribe + Miscellaneous',
            'Total Deductions Amount',
            'TDS @10%',
            'Total Amount after 10% TDS Deduction',
            'Total of Actual TDS',
            'TDS Paid in Advance',
            'Expenditure Incurred by CC',
            'Advance Paid to CC (90%)',
            'Total (Expenditure + Actual TDS)',
            'Net Payable to CC',
            'Net Receivable from CC',
            'Remarks'
        ];
    }

    public function styles(Worksheet $sheet)
    {
        // Apply styles to header row (A1:Z1 in Excel)
        $sheet->getStyle('A1:Z1')->applyFromArray([
            'font' => [
                'bold' => true,
            ],
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'startColor' => [
                    'rgb' => 'FFFF00', // Yellow background color
                ],
            ],
        ]);

        // Freeze the first row
        $sheet->freezePane('A2');

        // Loop through the rows to find "Total"
        foreach ($this->data as $index => $row) {
            if (in_array('Total', $row)) {
                $rowIndex = $index + 2; // +2 because Excel row index starts at 1 and header is at row 1
                $sheet->getStyle("A{$rowIndex}:Z{$rowIndex}")->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'color' => ['rgb' => '000000'], // White text
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'startColor' => ['rgb' => 'd9dede'], // Blue background
                    ],
                ]);
            }
        }
    }
}
