<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\User;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Auth;
use App\Services\MasterServices;
use Illuminate\Support\Facades\Response;
use Carbon\Carbon;

use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Str;
use App\Mail\EmailOtpSend;

class HomeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        return view('auth.home');
    }


    public function username(Request $request)
    {

        $validator = Validator::make($request->all(), ['username' => 'required|min:10|max:20']);
        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator);
        }

        $username = $request->username;

        $user = User::where('UserName', $username)->first();

        if (!$user) {
            return redirect()->back()->withErrors("User Not Found");
        }


        Session::put('guest_data', $username);
        return redirect()->route('login')->with('username', $username);
    }

    public function login(Request $request)
    {


        if ($request->isMethod('post')) {

            $validator = Validator::make($request->all(), ['password' => 'required']);
            if ($validator->fails()) {
                return redirect()->back()->withErrors($validator);
            }
            $credentials = [
                'username' => $request->username,
                'password' => $request->password
            ];

            if (Auth::attempt($credentials)) {
                MasterServices::customlog('login', 'login user');
                // Session::flash('success', 'Login successful!'); 
                return redirect()->intended('/dashboard');
            }

            return back()->withErrors([
                'username' => 'The provided credentials do not match our records.',
            ]);
        } elseif (Session::has('guest_data')) {
            return view('auth.login');
        } else {
            return redirect('home');
        }
    }

    public function ResetPassword(Request $request)
    {
        if ($request->isMethod('post')) {
            $NtaIdsArray = $request->NTAID ?? [];
            if (isset($NtaIdsArray) && !empty($NtaIdsArray)) {
                $NTAID = implode("','", $NtaIdsArray);
                // pr($NTAID);
                $venderData = DB::select("exec getCommonDataSP :select, :input, :table", [
                    'select' => "ID,MobileNo,UserName",
                    'input' => "WHERE UserName IN ('$NTAID')",
                    'table' => "FRONT_ERP_USER_AdminUser"
                ]);

                //prd($venderData);
                if (!empty($venderData)) {
                    foreach ($venderData as $val) {
                        $ID = $val->UserName;
                        $MobileNo = $val->MobileNo;
                        $pass = Hash::make($MobileNo); // Use Laravel's Hash facade for password hashing
                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "PasswordHash='$pass'",
                            'where' => "where UserName='$ID'",
                            'table' => "FRONT_ERP_USER_AdminUser",
                        ]);

                        $data[$ID] = [
                            'NTA ID' => $ID,
                            'Password' => $MobileNo,
                        ];
                    }

                    $uniqueData = array_values($data);
                    return $this->arrayToCsvDownload($uniqueData);
                }
            }
            return back()->with('error', 'Functionary ID is required!');
        }
        return back()->with('error', 'error in password reset!');
    }



    public function CreateCredentials()
    {
        $ExamProfileTbl = config('constants.ExamProfile');
        $RoleNameArray = config('constants.RoleName');
        $excludeMobiles = ['#N/A', ' ', 'NULL', '0', '#REF!'];
        $excludeObserversIDs = [' ', '0'];

        $venderData = DB::table($ExamProfileTbl)->select('Observer Type', 'Mobile', 'ObserverID', 'Email', 'Name')
            ->whereNotIn('Mobile', $excludeMobiles)
            ->whereNotIn('Observer Type', $excludeObserversIDs)
            ->whereNotIn('ObserverID', function ($query) {
                $query->select('UserName')->from('FRONT_ERP_USER_AdminUser');
            })->get();
        // prd($venderData);

        if ($venderData->isEmpty()) {
            return back()->with('warning', 'No data remains for credentials');
        }

        $data = [];
        $currentDate = Carbon::now();
        $authKey = 'K5F43sahm18gLZ8ZPbWe1mFnAOM1Ylc3';
        $isActive = 1;

        foreach ($venderData as $val) {
            $roleName = config("constants.RoleName.{$val->{'Observer Type'}}");
            $generatePass = trim($val->Mobile);
            $pass = Hash::make($generatePass);
            $roleID = $val->{'Observer Type'};

            DB::table('FRONT_ERP_USER_AdminUser')->insert([
                'Role_ID' => $roleID,
                'UserName' => $val->{'ObserverID'},
                'AuthKey' => $authKey,
                'PasswordHash' => $pass,
                'Email' => $val->Email,
                'MobileNo' => $val->Mobile,
                'IsActive' => $isActive,
                'CreatedDate' => $currentDate,
                'NTA_Unique_Admin_ID' => $val->{'ObserverID'},
                'FullName' => $val->Name,
                'isDefaultPassword' => 1,
            ]);

            $data[] = [
                'Name' => $val->Name,
                'Email' => $val->Email,
                'Mobile' => $val->Mobile,
                'NTA ID' => $val->{'ObserverID'},
                'Password' => $generatePass,
                'Role' => $roleName,
            ];
        }

        return $this->arrayToCsvDownload($data);
    }

    private function arrayToCsvDownload($array, $fileName = "ERP_Credentials.csv", $delimiter = ",")
    {
        $headers = [
            "Content-Type" => "text/csv",
            "Content-Disposition" => "attachment; filename={$fileName}",
        ];

        $callback = function () use ($array, $delimiter) {
            $handle = fopen('php://output', 'w');

            if (!empty($array)) {
                fputcsv($handle, array_keys($array[0]), $delimiter);
                foreach ($array as $row) {
                    fputcsv($handle, $row, $delimiter);
                }
            }

            fclose($handle);
        };

        return Response::stream($callback, 200, $headers);
    }


    public function forgotPassword(Request $request)
    {
        return view('auth.forgot-password');
    }

    public function userForgotPassword(Request $request)
    {
        // $validator = Validator::make($request->all(), [
        //     'username' => 'required|min:10|max:20',
        //     'Email' => ['required', 'email', 'regex:/^[\w\.]+@[a-zA-Z0-9\-]+\.[a-zA-Z]{2,6}$/'],
        // ]);
        // if ($validator->fails()) {
        //     return redirect()->back()->withErrors($validator);
        // }

        $request->validate([
            'username' => 'required|min:10|max:20',
            'Email' => ['required', 'email', 'regex:/^[\w\.]+@[a-zA-Z0-9\-]+\.[a-zA-Z]{2,6}$/'],
        ]);



        $username = $request->username;
        $adminemail = $request->Email;

        $user = User::where('UserName', $username)->where('Email', $adminemail)->first();
        if (!$user) {
            return redirect()->back()->withErrors("User Not Found");
            // return redirect()->route('home')->with('userNotFound', 'User Not Found');
        } else {

            $Name = $user->FullName ?? 'User';
            $userPassword = $user->PasswordHash;
            $token = Str::random(60);
            $token_expires_at = now()->addMinutes(10)->timestamp;
            $finalTokenNew = $token . '_' . $token_expires_at;

            $password = Crypt::encrypt($userPassword);
            $finalToken = $password . '_' . $finalTokenNew;
            $resetLink = url('/reset-password/' . $finalToken);

            if (!empty($username)) {
                DB::table('FRONT_ERP_USER_AdminUser')->where('UserName', $username)->where('Email', $adminemail)->update(['PasswordResetToken' => $finalToken]);
                $To = $adminemail;
                $TEMPLATE_UID = '66bb247747867';
                $PARAMS = array('CANDIDATENAME' => ucfirst($Name), 'PROJECTNAME' => 'Team National Testing Agency', 'LINK' => $resetLink);
                $sentEmail = resetPasswordMail($To, $TEMPLATE_UID, $PARAMS);

                if ($sentEmail) {
                    return redirect()->route('home')->with('success', 'A password reset link has been sent to your email. Please check your inbox');
                } else {
                    return back()->withErrors('Mail not end');
                }
            } else {
                return back()->withErrors('Something went to wrong');
            }
        }
    }

    public function showResetForm(Request $request, $token)
    {
        try {
            $stringUrl = explode('_', $token);
            $tokenPass = $stringUrl[0];
            $expiryTime = $stringUrl[2];
            $OldDecryptedpassword = Crypt::decrypt($tokenPass);
            $countToken = DB::table('FRONT_ERP_USER_AdminUser')->where('PasswordHash', $OldDecryptedpassword)->where('PasswordResetToken', $token)->count();
            if ($countToken > 0) {
                if (now()->timestamp > $expiryTime) {
                    return redirect()->route('home')->withErrors('The token has expired!');
                } else {
                    return view('auth.set_reset_password', compact('token'));
                }
            } else {
                return redirect()->route('home')->withErrors('The link not valid');
            }
        } catch (\Exception $e) {
            Log::error('Error while processing token: ' . $e->getMessage());
            return redirect()->route('home')->withErrors('An error occurred. Please try again later.');
        }
    }

    public function setFormgotPassword(Request $request)
    {
        $request->validate([
            'password' => 'required',
            'confirmpassword' => 'required|same:password',
        ]);
        $newPassword = trim($request->input('password'));
        $tokenPass = $request->input('token');

        $changePassword = Hash::make($newPassword);
        $updatePass = DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
            'updatefields' => "PasswordHash='$changePassword'",
            'where' => "where PasswordResetToken='$tokenPass'",
            'table' => "FRONT_ERP_USER_AdminUser",
        ]);
        if ($updatePass) {
            return redirect()->route('home')->with('success', 'Your password has been changed successfully.');
        } else {
            return back()->withErrors('something went to wrong');
        }
    }
}
