<?php

namespace App\Http\Controllers;

use App\Http\Requests\ChangePaswordRequest;
use Illuminate\Http\Request;
use DB;
use Auth;
use Redirect;
use Config;
use App\Models\VendorBankTbl;
use App\Models\VendorPanTbl;
use App\Models\VendorContactTbl;
use App\Models\User;
use Illuminate\Support\Facades\Hash;
use Session;
use Illuminate\Support\Facades\Storage;
use App\Services\MasterServices;



class UserController extends Controller
{
    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            if (auth()->guest()) {
                return redirect()->route('home');
            }

            return $next($request);
        });
    }


    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {

        try {
            $loginInfo = Auth::user();
            $VendorBankTbl = config('constants.VendorBankTbl');
            $VendorContactTbl = config('constants.VendorContactTbl');
            $VendorPanTbl = config('constants.VendorPanTbl');
            if ($loginInfo) {
                $userProfile = VendorBankTbl::where('No_', $loginInfo->UserName)->first();
                return view('user.view-profile', [
                    'userProfile' => $userProfile
                ]);
            }
        } catch (\Exception $e) {
            $message = $e->getMessage();
            return redirect()->back()->with('error', $message);
        }
    }

    public function edit()
    {
        try {
            $loginInfo = Auth::user();
            $VendorBankTbl = config('constants.VendorBankTbl');
            if ($loginInfo) {
                $userProfile = VendorBankTbl::where('No_', $loginInfo->UserName)->first();
                return view('user.edit-profile', [
                    'userProfile' => $userProfile
                ]);
            }
        } catch (\Exception $e) {
            $message = $e->getMessage();
            return redirect()->back()->with('error', $message);
        }
    }

    public function update(Request $request)
    {
        //echo $request->oldCheque;die;
        $validatedData = $request->validate([
            'FullName' => 'required',
            'MobileNo' => 'required',
            'Email' => 'required',
            'Address' => 'required',
            'BankName' => 'required',
            'Branch' => 'required',
            'IFSCCode' => 'required',
            'AccountNumber' => 'required',
            'AccountHolderName' => 'required',
            'DOB' => 'required',
            'PanNumber' => ['required', 'string', 'regex:/^[A-Z]{5}[0-9]{4}[A-Z]{1}$/'],
            'Cheque' => ($request->file('Cheque') || $request->oldCheque) ? 'nullable|file' : 'required|file|mimes:jpg,jpeg,png|max:2048',
            'ProfileImage' => 'file|mimes:jpg,jpeg,png|max:2048',
            'PANImage' => 'file|mimes:jpg,jpeg,png|max:2048',
        ], [
            'FullName.required' => 'FullName cannot be blank.',
            'MobileNo.required' => 'MobileNo cannot be blank.',
            'Email.required' => 'Email cannot be blank.',
            'Address.required' => 'Address cannot be blank.',
            'BankName.required' => 'Bank Name cannot be blank.',
            'Branch.required' => 'Branch cannot be blank.',
            'IFSCCode.required' => 'IFSC Code cannot be blank.',
            'AccountNumber.required' => 'Account Number cannot be blank.',
            'AccountHolderName.required' => 'Account Holder Name cannot be blank.',
            'DOB.required' => 'DOB cannot be blank.',
            'PanNumber.required' => 'Pan Number cannot be blank.',
            'Cheque.required' => 'Cancelled Cheque cannot be blank.',
        ]);
        try {
            $loginInfo = Auth::user();
            $VendorBankTbl = config('constants.VendorBankTbl');
            $VendorContactTbl = config('constants.VendorContactTbl');
            $VendorPanTbl = config('constants.VendorPanTbl');
            $ExamProfile = config('constants.ExamProfile');
            $UserAdminTable = 'FRONT_ERP_USER_AdminUser';

            if ($loginInfo) {
                $destinationPath = public_path('uploads/' . $loginInfo->UserName);                    
                if ($request->file('ProfileImage')) {
                    $file = $request->file('ProfileImage');
                    if (!file_exists($destinationPath)) {
                        mkdir($destinationPath, 0777, true);
                    }                
                    $fileName = time().'_.' . $file->getClientOriginalExtension();
                    $file->move($destinationPath, $fileName);
                    $profileImagePath = 'uploads/' . $loginInfo->UserName . '/' . $fileName;
                } else {
                    $profileImagePath = $request->oldProfileImage ? $request->oldProfileImage : 'NULL';
                }

                if ($request->file('PANImage')) {
                    $file = $request->file('PANImage');
                    if (!file_exists($destinationPath)) {
                        mkdir($destinationPath, 0777, true);
                    }                
                    $fileName = time().'_.' . $file->getClientOriginalExtension();
                    $file->move($destinationPath, $fileName);
                    $panImagePath = 'uploads/' . $loginInfo->UserName . '/' . $fileName;
                } else {
                    $panImagePath = $request->oldPANImage ? $request->oldPANImage : 'NULL';
                }

                if ($request->file('Cheque')) {
                    $file = $request->file('Cheque');
                    if (!file_exists($destinationPath)) {
                        mkdir($destinationPath, 0777, true);
                    }                
                    $fileName = time().'_.' . $file->getClientOriginalExtension();
                    $file->move($destinationPath, $fileName);
                    $chequeImagePath = 'uploads/' . $loginInfo->UserName . '/' . $fileName;
                } else {
                    $chequeImagePath = $request->oldCheque ? $request->oldCheque : '';
                }

                // $userUpdate = VendorBankTbl::where('No_', $loginInfo->UserName)->update([
                //     'Bank Name' => $request->BankName ? $request->BankName : null,
                //     'Branch' => $request->Branch ? $request->Branch : null,
                //     'IFSC Code' => $request->IFSCCode ? $request->IFSCCode : null,
                //     'Vendor Account No_' => $request->AccountNumber ? $request->AccountNumber : null,
                //     'Vendor D_O_B' => $request->DOB ? $request->DOB : null,
                //     'Profile Image' =>  $profileImagePath ?  $profileImagePath : null,
                //     'PAN Image Path' =>  $panImagePath ?  $panImagePath : null,
                //     'Cancelled Cheque Path' =>  $chequeImagePath ?  $chequeImagePath : null,
                // ]);
                $bankName = $request->BankName ? $request->BankName : null;
                $Branch = $request->Branch ? $request->Branch : null;
                $IFSCCode = $request->IFSCCode ? $request->IFSCCode : null;
                $VendorAccountNo_ = $request->AccountNumber ? $request->AccountNumber : null;
                $VendorD_O_B = $request->DOB ? $request->DOB : null;
                $ProfileImage =  $profileImagePath ?  $profileImagePath : null;
                $PANImagePath =  $panImagePath ?  $panImagePath : null;
                $CancelledChequePath =  $chequeImagePath ?  $chequeImagePath : null;
                $Address = $request->Address ? $request->Address : null;
                $MobileNo = $request->MobileNo ? $request->MobileNo : null;
                $Email = $request->Email ? $request->Email : null;
                $FullName = $request->FullName ? $request->FullName : null;
                $Panno = $request->PanNumber ? $request->PanNumber : null ; 
                $AccountHolderName = $request->AccountHolderName ? $request->AccountHolderName : null ; 

                $userUpdate = DB::statement(DB::raw("exec updateCommonDataSP :updatefields, :where, :table"), [
                    ':updatefields' => "[Bank Name] = '$bankName',Branch= '$Branch',[IFSC Code]= '$IFSCCode',[Vendor Account No_]= '$VendorAccountNo_',[Vendor D_O_B]= '$VendorD_O_B',[Profile Image]= '$ProfileImage',[PAN Image Path]= '$PANImagePath',[Cancelled Cheque Path]='$CancelledChequePath',[Account Holder Name] = '$AccountHolderName' ",
                    ':where' => "WHERE No_= '$loginInfo->UserName'",
                    ':table' => "[$VendorBankTbl]",
                ]);              

                $userContactUpdate = DB::statement(DB::raw("exec updateCommonDataSP :updatefields, :where, :table"), [
                    ':updatefields' =>  "Address = '$Address',[Mobile Phone No_] = '$MobileNo',[E-Mail] = '$Email',Name='$FullName'",
                    ':where' => "WHERE No_= '$loginInfo->UserName'",
                    ':table' => "[$VendorContactTbl]",
                ]);
                
                $userPanUpdate = DB::statement(DB::raw("exec updateCommonDataSP :updatefields, :where, :table"), [
                    ':updatefields' => "[P_A_N_ No_] =  '$Panno'",
                    ':where' => "WHERE [No_]= '$loginInfo->UserName'",
                    ':table' => "[$VendorPanTbl]",
                ]);

                $userProfileUpdate = DB::statement(DB::raw("exec updateCommonDataSP :updatefields, :where, :table"), [
                    ':updatefields' => "[Email] =  '$Email'",
                    ':where' => "WHERE [ObserverID]= '$loginInfo->UserName'",
                    ':table' => "[$ExamProfile]",
                ]);

                $userAdminUpdate = DB::statement(DB::raw("exec updateCommonDataSP :updatefields, :where, :table"), [
                    ':updatefields' => "[Email] =  '$Email'",
                    ':where' => "WHERE [UserName]= '$loginInfo->UserName'",
                    ':table' => "[$UserAdminTable]",
                ]);
              
                
            }
            MasterServices::customlog('Profile','User Update Profile.');
            return redirect()->route('view-profile')->with('success', 'Data updated successfully.');
        } catch (\Exception $e) {
            $message = $e->getMessage();
            dd($message);
            return redirect()->back()->with('error', $message);
        }
    }

    public function DeleteImg(Request $request, $id, $type)
    {
        $No_ = request('id');
        $type = request('type');
        if (!empty($type) || !empty($No_)) {


            if ($type == 'PAN Image Path') {
                $userUpdate = VendorBankTbl::where('No_', $No_)->update([
                    'PAN Image Path' => '',
                ]);
                MasterServices::customlog('Delete PAN Image','User Delete PAN Image');
                return redirect()->route('view-profile')->with('success', 'PAN Image deleted successfully.');
            }

            if ($type == 'Cancelled Cheque Path') {
                $userUpdate = VendorBankTbl::where('No_', $No_)->update([
                    'Cancelled Cheque Path' => '',
                ]);
                MasterServices::customlog('Cancelled Cheque Image','User Delete Cancelled Cheque Image');
                return redirect()->route('view-profile')->with('success', 'Cancelled Cheque deleted successfully.');
            }

            if ($type == 'Profile Image') {
                $userUpdate = VendorBankTbl::where('No_', $No_)->update([
                    'Profile Image' => '',
                ]);
                MasterServices::customlog('Profile Image','User Delete Profile Image');
                return redirect()->route('view-profile')->with('success', 'Profile Image deleted successfully.');
            }
        }


        return redirect()->back();
    }


    public function ChangePassword()
    {
    // prd(Auth::user());


        return view('user.change-password');

        // print_r($request->all());die;
        //$model = new User();

        // if ($this->request->isPost && $model->load(Yii::$app->request->post()) && $model->validate()) {
        //     // Check if the current password is correct


        //     if ($user->validatePassword($model->CurrentPassword)) {
        //         // Update the password
        //         $user->PasswordHash = Yii::$app->security->generatePasswordHash($model->NewPassword);
        //         $user->isDefaultPassword = 0;

        //         if ($user->save(false)) {
        //             Yii::$app->session->setFlash('success', 'Password changed successfully.');
        //             return $this->redirect(['index']);
        //         } else {
        //             Yii::$app->session->setFlash('error', 'Failed to change password.');
        //             return $this->redirect(Url::current());
        //         }
        //     } else {
        //         Yii::$app->session->setFlash('error', 'Incorrect current password.');
        //     }
        // }

    }



    public function updatePassword(ChangePaswordRequest $request)
    {
        $user = Auth::user();
        $newPassword = $request->input('NewPassword');
        $confirmPassword = $request->input('ConfirmPassword');
        $currentPassword = $request->input('CurrentPassword');

        // Check if new password and confirm password match
        if ($newPassword !== $confirmPassword) {
            return back()->withErrors(['ConfirmPassword' => 'The confirm password does not match the new password.'])->withInput();
        }

        // Check if the current password is correct
        if (!Hash::check($currentPassword, $user->password)) {
            return back()->withErrors(['CurrentPassword' => 'The current password is incorrect.'])->withInput();
        }

        // Update the user's password
       $changePassword = Hash::make($newPassword);
       DB::statement(DB::raw("exec updateCommonDataSP :updatefields, :where, :table"), [
        ':updatefields' => "password='$changePassword'",
        ':where' => "WHERE ID= '$user->ID'",
        ':table' => "FRONT_ERP_USER_AdminUser",
    ]);
       MasterServices::customlog('Update Password','User Change Password');
        return redirect()->route('dashboard')->with('success', 'Password updated successfully');
        
    }
    public function logout(Request $request)
    {
        MasterServices::customlog('logout','logout user');
        Auth::logout();
        Session::flush();
        Session::regenerateToken();
        Session::invalidate();
        return redirect('/home');
    }
}
