<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use App\Http\Controllers\OldReportController;
use Illuminate\Contracts\Session\Session as SessionSession;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Session;
use PDF;
use App\Http\Controllers\PdfDownloadController;


class AdminDashboardController extends Controller
{
    public function index()
    {
        $examMode = config('constants.Exam_Mode', []);
        $title = 'Dashboard';
        return view('admin.dashboard', ['examMode' => $examMode,'title'=>$title]);
    }

    public function superIndex()
    {
        $examMode = config('constants.Exam_Mode', []);
        $title = 'SuperAdmin Dashboard';
        return view('admin.superdashboard', ['examMode' => $examMode,'title'=>$title]);
    }

    public function serachExamByExamMode(Request $request)
    {
        $examMode = $request->post('examMode');
        $examTable = config('constants.ExamDates', []);
        $exams = [];
        $exams = DB::select(DB::raw("exec getCommonDataSP :select, :input, :table, :orderBy"), [
            'select' => '[Exam Type Id] as Exam_Type_Id,[Exam Name] as Exam_Name,[Exam Type] as ExamType',
            'input' => "WHERE Exam_Mode =  '{$examMode}' AND Is_Active=1",
            'table' => "[$examTable]",
            'orderBy' => 'ORDER BY [Exam Name] ASC'
        ]);

        return response()->json([
            'success' => true,
            'data' => $exams,
        ]);
    }

    public function getTotalCountBill(Request $request)
    {

        $CityCoordinatorOfflineTbl = config('constants.CityCoordinatorOfflineTbl');
        $TABillHeaderTbl = config('constants.TABillHeaderTbl');
        $CentreBillTbl = config('constants.CentreBillTbl');
        $ExamProfile = config('constants.ExamProfile');
        $examType = $request->post('examType');

        $tables = [
            $TABillHeaderTbl => ['role' => [5, 2, 3, 10],'table'=>'OBSERVER',  'type' => 'OBSERVER BILL', 'exam_type_column' => 'Exam Code'], // observer table
            $CentreBillTbl => ['role' => [14], 'table'=>'CENTRE','type' => 'CENTRE SUPERINTENDENT BILL', 'exam_type_column' => 'Exam Type'], // Centre Bill table
            $CityCoordinatorOfflineTbl => ['role' => [1], 'table'=>'CITY','type' => 'CITY COORDINATOR BILL', 'exam_type_column' => 'Exam Type'], // City Coordinator Bill table
        ];
        $data = [];
        foreach ($tables as $tableName => $info) {

            $examProfileCountsAll = DB::table($ExamProfile)
                ->selectRaw(' count(distinct(ObserverID)) as totalBill')
                ->where('ExamType', $examType)
                ->whereIn('Observer Type', $info['role'])
                ->where('Is Dummy', '0')
                ->first();
            $examProfileCountsUnSubmitted = DB::table($ExamProfile)
                ->selectRaw('count(distinct(ObserverID)) as totalBill')
                ->where('ExamType', $examType)
                ->whereIn('Observer Type', $info['role'])
                ->where('Is Dummy', '0')
                ->where('Submitted Documents', '0')
                ->first();

            $observerList = DB::table("$tableName as t1") // Alias for table
                ->join("$ExamProfile as t2", 't1.' . 'Observer ID', '=', 't2.ObserverID') // Fixed column format
                ->where('t1.' . $info['exam_type_column'], $examType)
                ->where('t2.' . 'Is Dummy', 0)
                ->whereIn('t2.' . 'Observer Type', $info['role'])
                ->where('t2.' . 'ExamType', $examType)
                ->selectRaw("DISTINCT (t1.[Observer ID]) as observerID")
                ->pluck('observerID')
                ->toArray();
            $billCounts = DB::table("$tableName")
                ->whereIn('Observer ID', $observerList)
                ->where($info['exam_type_column'], $examType)
                ->selectRaw("
                        SUM(CASE WHEN [Bill Status] = 0 THEN 1 ELSE 0 END) AS [open],
                        SUM(CASE WHEN [Bill Status] = 1 THEN 1 ELSE 0 END) AS [processing],
                        SUM(CASE WHEN [Bill Status] = 3 THEN 1 ELSE 0 END) AS [rejected],
                        SUM(CASE WHEN [Bill Status] = 5 THEN 1 ELSE 0 END) AS [completed]
                        ")
                ->first();

            if ($billCounts) {
                $data[] = [
                    "type" => $info['type'],
                    "table" => $info['table'],
                    "examType" =>  $examType,
                    "status" => [
                        "open" => (int)$billCounts->open,
                        "processing" => (int)$billCounts->processing,
                        "rejected" => (int)$billCounts->rejected,
                        "completed" => (int)$billCounts->completed,
                        "unsubmitted" => (int)$examProfileCountsUnSubmitted->totalBill,
                        "totalbill" => (int)$examProfileCountsAll->totalBill
                    ],
                    "labelName" => [
                        "open" => 'Open',
                        "processing" => 'In Process Bill',
                        "rejected" => 'Rejected',
                        "completed" => 'Payment Completed',
                        "unsubmitted" => 'Unsubmitted',
                        "totalbill" => 'Total Bill'
                    ],
                    "statusKey" => [
                        "open" => 0,
                        "processing" => 1,
                        "rejected" => 3,
                        "completed" => 5,
                        "unsubmitted" => 'UN',
                        "totalbill" => 'TOT'
                    ],
                ];
            } else {
                $data[] = [
                    "type" => $info['type'],
                    "table" => $info['table'],
                    "status" => [
                        "open" => 0,
                        "processing" => 0,
                        "rejected" => 0,
                        "completed" => 0,
                        "unsubmitted" => 0,
                        "totalbill" => 0,
                    ],
                ];
            }
        }
        return response()->json([
            'success' => true,
            'data' => $data,
        ]);
    }

    public function getAllBilldata()
    {

        $CityCoordinatorOfflineTbl = config('constants.CityCoordinatorOfflineTbl');
        $TABillHeaderTbl = config('constants.TABillHeaderTbl');
        $CentreBillTbl = config('constants.CentreBillTbl');
        $ExamProfile = config('constants.ExamProfile');
        $ExamDates = config('constants.ExamDates');
        // Validate configuration constants
        if (empty($CityCoordinatorOfflineTbl) || empty($ExamProfile)) {
            return response()->json(['error' => 'Configuration constants are missing or invalid'], 400);
        }
        // Escape table names for SQL Server
        $CityCoordinatorOfflineTbl = '[' . str_replace(['[', ']'], '', $CityCoordinatorOfflineTbl) . ']';
        $CentreBillTbl = '[' . str_replace(['[', ']'], '', $CentreBillTbl) . ']';
        $ExamProfile = '[' . str_replace(['[', ']'], '', $ExamProfile) . ']';
        $TABillHeaderTbl = '[' . str_replace(['[', ']'], '', $TABillHeaderTbl) . ']';
        $ExamDates = '[' . str_replace(['[', ']'], '', $ExamDates) . ']';

        try {
            $query = "WITH BillHeader AS (
                SELECT [Reference No_] ReferenceNo, [Observer ID] ObserverID, [Exam ID] ExamID, [Bill Status]  BillStatus, 'CITY' as examtable FROM $CityCoordinatorOfflineTbl
                UNION ALL
                SELECT [Reference No_] ReferenceNo, [Observer ID] ObserverID, [Exam ID] ExamID, [Bill Status]  BillStatus, 'CENTRE' as examtable FROM $CentreBillTbl
                UNION ALL
                SELECT [Reference No_] ReferenceNo, [Observer ID] ObserverID, [Exam_ID] ExamID, [Bill Status]  BillStatus, 'OBSERVER' as examtable FROM $TABillHeaderTbl
            ),
            RankedData AS (
                SELECT
                    bh.ReferenceNo,
                    bh.ObserverID,
                    ex.Name AS ObserverName,
                    da.[Exam Name] AS ExamID,
                    bh.BillStatus,
                    ex.[Observer Type],
                    ex.Mobile,
                    ex.Email,
                    bh.examtable,
                    ex.ExamType,
                    ROW_NUMBER() OVER (PARTITION BY bh.ReferenceNo ORDER BY bh.ObserverID DESC) AS RowNum
                FROM BillHeader bh
                INNER JOIN $ExamProfile ex ON ex.ObserverID = bh.ObserverID
                INNER JOIN $ExamDates da ON da.[Exam Type Id] = bh.ExamID
            )
            SELECT
                ReferenceNo,
                ObserverID,
                ObserverName,
                ExamID,
                CASE
                    WHEN BillStatus = 0 THEN 'Open'
                    WHEN BillStatus = 1 THEN 'Paid'
                    WHEN BillStatus = 2 THEN 'Pending'
                    ELSE '-'
                END AS BillStatus,
                CASE
                    WHEN [Observer Type] = 1 THEN 'City Coordinator'
                    WHEN [Observer Type] = 14 THEN 'Centre Superintendent'
                    WHEN [Observer Type] = 5 THEN 'Observer'
                    ELSE '-'
                END AS Role,
                Mobile,
                Email,
                examtable,
                ExamType
            FROM RankedData
            WHERE RowNum = 1
            ORDER BY ReferenceNo DESC";

            $AllBilldata = DB::select(DB::raw($query));
            // prd($AllBilldata);
            return response()->json($AllBilldata, 200);
        } catch (\Exception $e) {
            return response()->json([
                'error' => 'An error occurred while fetching data',
                'details' => $e->getMessage()
            ], 500);
        }
    }
    public function billDetails(Request $request)
    {
        try {
            // Fetching table names from config constants
            $CityCoordinatorOfflineTbl = config('constants.CityCoordinatorOfflineTbl');
            $TABillHeaderTbl = config('constants.TABillHeaderTbl');
            $CentreBillTbl = config('constants.CentreBillTbl');
            $VendorBankTbl = config('constants.VendorBankTbl');
            $VendorPanTbl = config('constants.VendorPanTbl');
            $VendorContactTbl = config('constants.VendorContactTbl');
            $ExamProfile = config('constants.ExamProfile');
            $TABillLocalTbl = config('constants.TABillLocalTbl');
            $TABillHotelTbl = config('constants.TABillHotelTbl');
            $TABillOutstationTbl = config('constants.TABillOutstationTbl');
            $examTable = config('constants.ExamDates');

            // Clean table names by removing square brackets
            $CityCoordinatorOfflineTbl = '[' . str_replace(['[', ']'], '', $CityCoordinatorOfflineTbl) . ']';
            $TABillHeaderTbl = '[' . str_replace(['[', ']'], '', $TABillHeaderTbl) . ']';
            $CentreBillTbl = '[' . str_replace(['[', ']'], '', $CentreBillTbl) . ']';
            $VendorBankTbl = '[' . str_replace(['[', ']'], '', $VendorBankTbl) . ']';
            $VendorPanTbl = '[' . str_replace(['[', ']'], '', $VendorPanTbl) . ']';
            $VendorContactTbl = '[' . str_replace(['[', ']'], '', $VendorContactTbl) . ']';
            $ExamProfile = '[' . str_replace(['[', ']'], '', $ExamProfile) . ']';
            $TABillLocalTbl = '[' . str_replace(['[', ']'], '', $TABillLocalTbl) . ']';
            $TABillHotelTbl = '[' . str_replace(['[', ']'], '', $TABillHotelTbl) . ']';
            $TABillOutstationTbl = '[' . str_replace(['[', ']'], '', $TABillOutstationTbl) . ']';
            $examTable = '[' . str_replace(['[', ']'], '', $examTable) . ']';

            // Validate incoming request data
            $validated = $request->validate([
                'referenceNo' => 'required|string',
                'observerId' => 'required|string',
                'nextval' => 'required|string',
                'examtable' => 'required|string',
                'status' => 'required|string',
                'type' => 'required|string',
                'verify' => 'required|string',
            ]);
            $referenceNo = $validated['referenceNo'];
            $observerId = $validated['observerId'];
            $nextval = $validated['nextval'];
            $billTable = $validated['examtable'];
            $Billstatus = $validated['status'];
            $examType = $validated['type'];
            $verify = $validated['verify'];

            $TABillLocalData=[];
            $TABillHotelData=[];
            $TABillOutstationData=[];
            $data = null;
            // prd($validated);
            // update bill status
            $success=0;
            if($verify !='no')
            {

                if($verify=='rejected')
                {
                    if ($billTable=='CITY') {
                    $result= DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Bill Status]='3'",
                            'where' => "where  [Exam Code]='$examType' and [Reference No_] ='$referenceNo' ",
                            'table' => "$CityCoordinatorOfflineTbl",
                        ]);
                   $success = $result ? 1 : 0;
                    }
                    else if ($billTable=='CENTRE') {
                       $result= DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Bill Status]='3'",
                            'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                            'table' => "$CentreBillTbl",
                        ]);
                    $success = $result ? 1 : 0;
                    }
                    else if ($billTable=='OBSERVER') {
                        $result=  DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Bill Status]='3'",
                            'where' => "where  [Exam Code]='$examType' and [Reference No_] ='$referenceNo' ",
                            'table' => "$TABillHeaderTbl",
                        ]);
                        $success = $result ? 1 : 0;
                        $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Submitted Documents]='0'",
                            'where' => "where  [ExamType]='$examType' and [ObserverID] ='$observerId'",
                            'table' => "$ExamProfile",
                        ]);
                        $success = $result ? 1 : 0;
                    }
                }else if($verify=='approved')
                {

                    if ($billTable=='CITY') {
                        $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Bill Status]='4'",
                            'where' => "where  [Exam Type]='$examType' and [Reference No_]='$referenceNo'",
                            'table' => "$CityCoordinatorOfflineTbl",
                        ]);
                        $success = $result ? 1 : 0;
                        }
                        else if ($billTable=='CENTRE') {
                            $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[Bill Status]='4'",
                                'where' => "where  [Exam Type]='$examType' and [Reference No_]='$referenceNo'",
                                'table' => "$CentreBillTbl",
                            ]);
                            $success = $result ? 1 : 0;
                        }
                        else if ($billTable=='OBSERVER') {
                            $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[Bill Status]='4'",
                                'where' => "where  [Exam Code]='$examType' and [Reference No_]='$referenceNo'",
                                'table' => "$TABillHeaderTbl",
                            ]);
                            $success = $result ? 1 : 0;
                        }
                }else if($verify==='verifiedYes')
                {

                    if ($billTable=='CITY') {
                        $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Bill Status]='1'",
                            'where' => "where  [Exam Type]='$examType' and [Reference No_]='$referenceNo'",
                            'table' => "$CityCoordinatorOfflineTbl",
                        ]);
                        $success = $result ? 1 : 0;
                        }
                        else if ($billTable=='CENTRE') {
                            $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[Bill Status]='1'",
                                'where' => "where  [Exam Type]='$examType' and [Reference No_]='$referenceNo'",
                                'table' => "$CentreBillTbl",
                            ]);
                            $success = $result ? 1 : 0;
                        }
                        else if ($billTable=='OBSERVER') {

                            $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[Bill Status]='1'",
                                'where' => "where  [Exam Code]='$examType' and [Reference No_]='$referenceNo'",
                                'table' => "$TABillHeaderTbl",
                            ]);
                            $success = $result ? 1 : 0;
                        }
                }else if($verify=='complete')
                {
                    if ($billTable=='CITY') {
                        $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[Bill Status]='5'",
                            'where' => "where  [Exam Type]='$examType' and [Reference No_]='$referenceNo'",
                            'table' => "$CityCoordinatorOfflineTbl",
                        ]);
                        $success = $result ? 1 : 0;
                        }
                        else if ($billTable=='CENTRE') {
                            $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[Bill Status]='5'",
                                'where' => "where  [Exam Type]='$examType' and [Reference No_]='$referenceNo'",
                                'table' => "$CentreBillTbl",
                            ]);
                            $success = $result ? 1 : 0;
                        }
                        else if ($billTable=='OBSERVER') {
                            $result=DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[Bill Status]='5'",
                                'where' => "where  [Exam Code]='$examType' and [Reference No_]='$referenceNo'",
                                'table' => "$TABillHeaderTbl",
                            ]);
                            $success = $result ? 1 : 0;
                        }
                }
            }
            // end update

            if($nextval=='yes')
            {

                if ($Billstatus == 1) {
                    $status = "AND [Bill Status] IN ('1','2','4') AND  [Reference No_] !='$referenceNo' ORDER BY NEWID() ";
                }else{
                    $status = "AND [Bill Status]='$Billstatus' AND  [Reference No_] !='$referenceNo'  ORDER BY NEWID()";

                }
                if ($billTable=='CITY') {
                $query = "select TOP 1 * from $CityCoordinatorOfflineTbl where [Exam Type]='$examType' $status";
                $data = DB::select(DB::raw($query));
                $viewLoad = 'ccfields';


                }
                else if ($billTable=='CENTRE') {
                    $query = "select TOP 1 * from $CentreBillTbl where [Exam Type]='$examType' $status";
                    $data = DB::select(DB::raw($query));
                    $viewLoad = 'csfields';
                }

                else if ($billTable=='OBSERVER') {
                    $query = "select TOP 1 * from $TABillHeaderTbl where [Exam Code]='$examType' $status";
                    $data = DB::select(DB::raw($query));
                    $viewLoad = 'observerfields';

                    $observerId=$data[0]->{'Observer ID'};
                    $referenceNo=$data[0]->{'Reference No_'};

                    $query = "select * from $TABillLocalTbl where [Reference No_]='$referenceNo'";
                    $TABillLocalData = DB::select(DB::raw($query));
                    $query = "select * from $TABillHotelTbl where [Reference No_]='$referenceNo'";
                    $TABillHotelData = DB::select(DB::raw($query));
                    $query = "select * from $TABillOutstationTbl where [Reference No_]='$referenceNo'";
                    $TABillOutstationData = DB::select(DB::raw($query));
                }

                    $observerId=$data[0]->{'Observer ID'};
                    $referenceNo=$data[0]->{'Reference No_'};

            }else {

                $query = "select * from $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                $data = DB::select(DB::raw($query));
                $viewLoad = 'ccfields';


                if (empty($data)) {
                    $query = "select * from $CentreBillTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                    $data = DB::select(DB::raw($query));
                    $viewLoad = 'csfields';
                }

                if (empty($data)) {

                    $query = "select * from $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                    $data = DB::select(DB::raw($query));
                    $viewLoad = 'observerfields';

                    $query = "select * from $TABillLocalTbl where [Reference No_]='$referenceNo'";
                    $TABillLocalData = DB::select(DB::raw($query));
                    $query = "select * from $TABillHotelTbl where [Reference No_]='$referenceNo'";
                    $TABillHotelData = DB::select(DB::raw($query));
                    $query = "select * from $TABillOutstationTbl where [Reference No_]='$referenceNo'";
                    $TABillOutstationData = DB::select(DB::raw($query));
                }
            }
            // prd($TABillLocalData);
            // If no data was found from any of the tables
            if (empty($data)) {
                return response()->json(['error' => 'No records found for the provided reference number and observer ID.'], 404);
            }

            $emode = "select Exam_Mode from $examTable where [Exam Type]='$examType'";
            $exam_mode = DB::select(DB::raw($emode));

            $examMode=$exam_mode[0]->{'Exam_Mode'};

            if($examMode)
            {
                if($examMode=='CBT')
                {
                   $Mode='Online';
                }elseif($examMode=='PNP')
                {
                    $Mode='Offline';
                }
            }
            $query = "select * from $VendorBankTbl where [No_]='$observerId'";
            $VendorBank = DB::select(DB::raw($query));
            $query = "select [P_A_N_ No_] from $VendorPanTbl where [No_]='$observerId'";
            $VendorPan = DB::select(DB::raw($query));
            $query = "select * from $VendorContactTbl where [No_]='$observerId'";
            $VendorContact = DB::select(DB::raw($query));
            $query = "select [Observer Type] from $ExamProfile where [ObserverID]='$observerId' and [ExamType]='$examType'";
            $ExamProfile = DB::select(DB::raw($query));

            $VendorBank=$VendorBank[0]??[];
            $VendorPan=$VendorPan[0]??[];
            $VendorContact=$VendorContact[0]??[];
            $data=$data[0]??[];
            $role=config('constants.RoleName.'.$ExamProfile[0]->{'Observer Type'});
            $referenceNo=$data->{'Reference No_'};
            $BillStatus=$data->{'Bill Status'};
            $Role_ID=auth()->user()->Role_ID;

            $htmlView = view('admin.'.$viewLoad, compact('data', 'VendorBank', 'VendorPan', 'VendorContact', 'role', 'examType', 'Role_ID','billTable','Billstatus','TABillLocalData', 'TABillHotelData', 'TABillOutstationData', 'Mode', 'referenceNo'))->render();

            // Return JSON with both the view and additional data
            return response()->json([
                'html' => $htmlView,  // This contains the full view
                'role' => $role,
                'Mode' => $Mode,
                'Role_ID' => $Role_ID,
                'BillStatus' => $BillStatus,
                'updateStatus'=>$success,
                'referenceNo' => $referenceNo,
            ]);

        } catch (\Exception $e) {
            // Handle any errors
            return response()->json(['error' => 'Query execution failed: ' . $e->getMessage()], 500);
        }
    }

    public function getListingByLabel(Request $request)
    {
        // Validate the input parameters
        $validated = $request->validate([
            'label' => 'required|string',
            'type' => 'required|string',
            'status' => 'required|string',
            'examType' => 'required|string',
        ]);

        // Extract values from validated request
        $label = $validated['label'];
        $type = $validated['type'];
        $examType = $validated['examType'];

        // Set status and condition based on the input status
        $status = '';
        $THERCONDITION = '';
        if ($validated['status'] == 1) {
            $status = "AND CCTbl.[Bill Status] IN ('1','2','4')";
        } elseif ($validated['status'] == 'UN') {
            $THERCONDITION = "AND EX.[Submitted Documents] = '0'";
            $status = '';
        } elseif ($validated['status'] == 'TOT') {
            $status = '';
            $THERCONDITION = '';
        } else {
            $status = 'AND CCTbl.[Bill Status] IN (' . $validated['status'] . ')';
        }


        // Get table names from config constants
        $CityCoordinatorOfflineTbl = config('constants.CityCoordinatorOfflineTbl');
        $TABillHeaderTbl = config('constants.TABillHeaderTbl');
        $CentreBillTbl = config('constants.CentreBillTbl');
        $ExamProfile = config('constants.ExamProfile');

        // Sanitize table names to avoid issues with square brackets
        $sanitizeTableName = function ($table) {
            return '[' . str_replace(['[', ']'], '', $table) . ']';
        };

        // Clean the table names
        $CityCoordinatorOfflineTbl = $sanitizeTableName($CityCoordinatorOfflineTbl);
        $TABillHeaderTbl = $sanitizeTableName($TABillHeaderTbl);
        $CentreBillTbl = $sanitizeTableName($CentreBillTbl);
        $ExamProfile = $sanitizeTableName($ExamProfile);

        // Initialize default query components
        $SELECT = '';
        $WHERE = '';
        $TABLE = '';

        // Build query based on type
        switch ($type) {
            case 'CITY COORDINATOR BILL':
                if ($label == 'Unsubmitted') {
                    $SELECT = " DISTINCT '' as ReferenceNo,
                            ObserverID as ObserverID,
                            Name as ObserverName,
                            ExamType as ExamID,
                            Mobile,
                            BillType,
                            CENTNO as Centno,
                            [Observer Type] as [role],
                            Email";
                    $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0 AND [Submitted Documents]=0  AND  [Observer Type]  in(1) ";
                    $TABLE = "$ExamProfile";
                } else if ($label == 'Total Bill') {
                    $SELECT = " DISTINCT '' as ReferenceNo,
                            ObserverID as ObserverID,
                            Name as ObserverName,
                            ExamType as ExamID,
                            Mobile,
                            BillType,
                            CENTNO as Centno,
                            [Observer Type] as [role],
                            Email";
                    $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0  AND  [Observer Type]  in(1)";
                    $TABLE = "$ExamProfile";
                } else {
                    $SELECT = 'DISTINCT
                            CCTbl.[Reference No_] as ReferenceNo,
                            EX.[ObserverID] as ObserverID,
                            EX.[Name] as ObserverName,
                            EX.[ExamType] as ExamID,
                            EX.Mobile,
                            EX.BillType,
                            EX.CENTNO as Centno,
                            EX.[Observer Type] as [role],
                            EX.Email';
                    $WHERE = "INNER JOIN $CityCoordinatorOfflineTbl as CCTbl ON EX.[ObserverID] = CCTbl.[Observer ID] AND EX.[ExamType] = CCTbl.[Exam Type]
                           WHERE EX.[ExamType] = '$examType' $status AND EX.[Is Dummy] = 0 $THERCONDITION order by CCTbl.[Reference No_]";
                    $TABLE = "$ExamProfile AS EX";
                }
                break;

            case 'OBSERVER BILL':
                if ($label == 'Unsubmitted') {
                    $SELECT = " DISTINCT '' as ReferenceNo,
                            ObserverID as ObserverID,
                            Name as ObserverName,
                            ExamType as ExamID,
                            Mobile,
                            BillType,
                            CENTNO as Centno,
                            [Observer Type] as [role],
                            Email";
                    $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0 AND [Submitted Documents]=0  AND  [Observer Type]  in(5, 2, 3, 10)";
                    $TABLE = "$ExamProfile";
                } else if ($label == 'Total Bill') {
                    $SELECT = " DISTINCT '' as ReferenceNo,
                            ObserverID as ObserverID,
                            Name as ObserverName,
                            ExamType as ExamID,
                            Mobile,
                            BillType,
                            CENTNO as Centno,
                            [Observer Type] as [role],
                            Email";
                    $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0  AND  [Observer Type]  in(5, 2, 3, 10)";
                    $TABLE = "$ExamProfile";
                } else {
                    $SELECT = 'DISTINCT
                            CCTbl.[Reference No_] as ReferenceNo,
                            EX.[ObserverID] as ObserverID,
                            EX.[Name] as ObserverName,
                            EX.[ExamType] as ExamID,
                            EX.Mobile,
                            EX.BillType,
                            EX.CENTNO as Centno,
                            EX.[Observer Type] as [role],
                            EX.Email';
                    $WHERE = "INNER JOIN $TABillHeaderTbl as CCTbl ON EX.[ObserverID] = CCTbl.[Observer ID] AND EX.[ExamType] = CCTbl.[Exam Code]
                           WHERE EX.[ExamType] = '$examType' $status AND EX.[Is Dummy] = 0 $THERCONDITION order by [Reference No_]";
                    $TABLE = "$ExamProfile AS EX";
                }
                break;

            case 'CENTRE SUPERINTENDENT BILL':
                if ($label == 'Unsubmitted') {
                    $SELECT = " DISTINCT '' as ReferenceNo,
                            ObserverID as ObserverID,
                            Name as ObserverName,
                            ExamType as ExamID,
                            Mobile,
                            BillType,
                            CENTNO as Centno,
                            [Observer Type] as [role],
                            Email";
                    $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0 AND [Submitted Documents]=0  AND  [Observer Type]  in(14)";
                    $TABLE = "$ExamProfile";
                } else if ($label == 'Total Bill') {
                    $SELECT = " DISTINCT '' as ReferenceNo,
                            ObserverID as ObserverID,
                            Name as ObserverName,
                            ExamType as ExamID,
                            Mobile,
                            BillType,
                            CENTNO as Centno,
                            [Observer Type] as [role],
                            Email";
                    $WHERE = " WHERE [ExamType] = '$examType' AND [Is Dummy] = 0  AND  [Observer Type]  in(14)";
                    $TABLE = "$ExamProfile";
                } else {
                    $SELECT = 'DISTINCT
                        CCTbl.[Reference No_] as ReferenceNo,
                        EX.[ObserverID] as ObserverID,
                        EX.[Name] as ObserverName,
                        EX.[ExamType] as ExamID,
                        EX.Mobile,
                        EX.BillType,
                        EX.CENTNO as Centno,
                        EX.[Observer Type] as [role],
                        EX.Email';
                    $WHERE = "INNER JOIN $CentreBillTbl as CCTbl ON EX.[ObserverID] = CCTbl.[Observer ID] AND EX.[ExamType] = CCTbl.[Exam Type]
                       WHERE EX.[ExamType] = '$examType' $status AND EX.[Is Dummy] = 0 $THERCONDITION order by CCTbl.[Reference No_]";
                    $TABLE = "$ExamProfile AS EX";
                }
                break;

            default:
                return response()->json(['error' => 'Invalid bill type'], 400);
        }

        // Execute the query using parameterized binding for stored procedure
        // Use the 'DB::select' method for executing the query.
        try {
            // echo $SELECT,"<br>";
            // echo $WHERE,"<br>";
            // echo $TABLE;
            // die;
            $data = DB::select(DB::raw("EXEC getCommonDataSP :select, :input, :table"), [
                'select' => $SELECT,
                'input' => $WHERE,
                'table' => $TABLE,
                // 'orderBy' => 'ORDER BY [Reference No_] DESC',
            ]);
        } catch (\Exception $e) {
            return response()->json(['error' => 'Query execution failed: ' . $e->getMessage()], 500);
        }
        // Return the result as a JSON response
        return response()->json($data);
    }

    public function updateBillDeatils(Request $request)
    {

        try {
            // Fetching table names from config constants
            $CityCoordinatorOfflineTbl = config('constants.CityCoordinatorOfflineTbl');
            $TABillHeaderTbl = config('constants.TABillHeaderTbl');
            $CentreBillTbl = config('constants.CentreBillTbl');
            $VendorBankTbl = config('constants.VendorBankTbl');
            $VendorPanTbl = config('constants.VendorPanTbl');
            $VendorContactTbl = config('constants.VendorContactTbl');
            $ExamProfile = config('constants.ExamProfile');
            $TABillLocalTbl = config('constants.TABillLocalTbl');
            $TABillHotelTbl = config('constants.TABillHotelTbl');
            $TABillOutstationTbl = config('constants.TABillOutstationTbl');
            $examTable = config('constants.ExamDates');

            // Clean table names by removing square brackets
            $CityCoordinatorOfflineTbl = '[' . str_replace(['[', ']'], '', $CityCoordinatorOfflineTbl) . ']';
            $TABillHeaderTbl = '[' . str_replace(['[', ']'], '', $TABillHeaderTbl) . ']';
            $CentreBillTbl = '[' . str_replace(['[', ']'], '', $CentreBillTbl) . ']';
            $VendorBankTbl = '[' . str_replace(['[', ']'], '', $VendorBankTbl) . ']';
            $VendorPanTbl = '[' . str_replace(['[', ']'], '', $VendorPanTbl) . ']';
            $VendorContactTbl = '[' . str_replace(['[', ']'], '', $VendorContactTbl) . ']';
            $ExamProfile = '[' . str_replace(['[', ']'], '', $ExamProfile) . ']';
            $TABillLocalTbl = '[' . str_replace(['[', ']'], '', $TABillLocalTbl) . ']';
            $TABillHotelTbl = '[' . str_replace(['[', ']'], '', $TABillHotelTbl) . ']';
            $TABillOutstationTbl = '[' . str_replace(['[', ']'], '', $TABillOutstationTbl) . ']';
            $examTable = '[' . str_replace(['[', ']'], '', $examTable) . ']';

            // Validate incoming request data
            $validated = $request->validate([
                'referenceNo' => 'required|string',
                'observerId' => 'required|string',
                'examtable' => 'required|string',
                'status' => 'required|string',
                'type' => 'required|string',
                'updateId' => 'required|string',
                'getupdate' => 'nullable|string',

            ]);
            $referenceNo = $validated['referenceNo'];
            $observerId = $validated['observerId'];
            $billTable = $validated['examtable'];
            $Billstatus = $validated['status'];
            $examType = $validated['type'];
            $updateId = $validated['updateId'];
            $updateValue = $validated['getupdate'];
            // print_R($validated);
            // die;
            $TABillLocalData=[];
            $TABillHotelData=[];
            $TABillOutstationData=[];
            $data = null;

            // update bill status
            $columnName=str_replace('###',' ',$updateId);

            $success='Saved';

                if ($billTable=='CITY') {
                   if(in_array($columnName,['Remark','Rejection Remark','No_ of Candidate Alloted NTA'])){
                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                        'updatefields' => "[$columnName]='$updateValue'",
                        'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                        'table' => "$CityCoordinatorOfflineTbl",
                    ]);
                    }else if(in_array($columnName,['Advance Payment','TDS_Amount','Advance TDS Deduction'])){
                        $query = "select [$columnName],[Net Amount Payable],[Total Deduction Amount] from
                        $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                        $getDetail = DB::select(DB::raw($query));
                        $deduction=$getDetail[0]->{$columnName};
                        $NetAmountPayable=$getDetail[0]->{'Net Amount Payable'};
                        $TotalDeductionAmount=$getDetail[0]->{'Total Deduction Amount'};
                        $NetAmountPayable=($NetAmountPayable + $deduction) - ($updateValue);
                        $NetPayableAfterDeduction= $NetAmountPayable - $TotalDeductionAmount;

                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[$columnName]='$updateValue',[Net Amount Payable]='$NetAmountPayable',[Net Payable After Deduction]='$NetPayableAfterDeduction'",
                                'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                                'table' => "$CityCoordinatorOfflineTbl",
                            ]);

                    }
                    else{

                        $query = "select [$columnName],[Total Deduction Amount],[Net Amount Payable] from
                        $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                        $getDetail = DB::select(DB::raw($query));
                        $deduction=$getDetail[0]->{$columnName};
                        $TotalDeductionAmount=$getDetail[0]->{'Total Deduction Amount'};
                        $TotalDeductionAmount=($TotalDeductionAmount-$deduction) + $updateValue;
                        $NetPayableAmount=$getDetail[0]->{'Net Amount Payable'};
                        $NetPayableAfterDeduction=$NetPayableAmount - ($TotalDeductionAmount);

                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[$columnName]='$updateValue',[Total Deduction Amount]='$TotalDeductionAmount',[Net Payable After Deduction]='$NetPayableAfterDeduction'",
                                'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                                'table' => "$CityCoordinatorOfflineTbl",
                            ]);
                    }

                    }
                    else if ($billTable=='CENTRE') {
                        if(in_array($columnName,['Remarks','Rejection Remark','Day1 Count','Total Candidate Count Exam'])){
                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                        'updatefields' => "[$columnName]='$updateValue'",
                            'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                            'table' => "$CentreBillTbl",
                        ]);
                        }else if(in_array($columnName,['Amount Rec_ in Advance'])){
                            $query = "select [$columnName],[Net Amount Payable],[Total Deductions Amount] from
                            $CentreBillTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                            $getDetail = DB::select(DB::raw($query));
                            $deduction=$getDetail[0]->{$columnName};
                            $NetAmountPayable=$getDetail[0]->{'Net Amount Payable'};
                            $TotalDeductionsAmount=$getDetail[0]->{'Total Deductions Amount'};
                            $NetAmountPayable=($NetAmountPayable + $deduction) - $updateValue;
                            $NetPayableAfterDeduction=$NetAmountPayable - $TotalDeductionsAmount;

                            DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[$columnName]='$updateValue',[Net Amount Payable]='$NetAmountPayable',[Net Payable After Deductions]='$NetPayableAfterDeduction'",
                                    'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                                    'table' => "$CentreBillTbl",
                                ]);

                        }
                        else{
                            $query = "select [$columnName],[Total Deductions Amount],[Net Amount Payable] from
                            $CentreBillTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                            $getDetail = DB::select(DB::raw($query));
                            $deduction=$getDetail[0]->{$columnName};
                            $TotalDeductionAmount=$getDetail[0]->{'Total Deductions Amount'};
                            $TotalDeductionAmount=($TotalDeductionAmount-$deduction) + $updateValue;
                            $NetPayableAmount=$getDetail[0]->{'Net Amount Payable'};
                            $NetPayableAfterDeduction=$NetPayableAmount - ($TotalDeductionAmount);

                            DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[$columnName]='$updateValue',[Total Deductions Amount]='$TotalDeductionAmount',[Net Payable After Deductions]='$NetPayableAfterDeduction'",
                                    'where' => "where  [Exam Type]='$examType' and [Reference No_] ='$referenceNo' ",
                                    'table' => "$CentreBillTbl",
                                ]);
                        }

                    }
                    else if ($billTable=='OBSERVER') {
                        if(in_array($columnName,['Remark','Rejection Remark','CreatedDate','Completed Date'])){
                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                        'updatefields' => "[$columnName]='$updateValue'",
                            'where' => "where  [Exam Code]='$examType' and [Reference No_] ='$referenceNo' ",
                            'table' => "$TABillHeaderTbl",
                        ]);
                    }else if( $columnName=='P_A_N_ No_')
                        {
                            DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                                'updatefields' => "[$columnName]='$updateValue'",
                                    'where' => "where  [No_] ='$observerId' ",
                                    'table' => "$VendorPanTbl",
                                ]);
                    }else if(in_array($columnName,['Advance Payment']))
                    {
                        $query = "select [Net Payable Amount],[Claimed Amount],[TDS Amount],[Total Deduction Amount] from
                        $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                        $getDetail = DB::select(DB::raw($query));
                        $ClaimedAmount=$getDetail[0]->{'Claimed Amount'};
                        $TDSAmount=$getDetail[0]->{'TDS Amount'};
                        $NetPayableAmount=$getDetail[0]->{'Net Payable Amount'};
                        $NetPayableAmount=$ClaimedAmount -($TDSAmount+$updateValue);
                        $TotalDeductionAmount=$getDetail[0]->{'Total Deduction Amount'};
                        $NetPayableAfterDeduction=$NetPayableAmount - ($TotalDeductionAmount);

                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[$columnName]='$updateValue',[Net Payable Amount]='$NetPayableAmount',[Net Payable After Deduction]='$NetPayableAfterDeduction'",
                                'where' => "where  [Exam Code]='$examType' and [Reference No_] ='$referenceNo' ",
                                'table' => "$TABillHeaderTbl",
                            ]);
                    }else
                    {
                        $query = "select [$columnName],[Total Deduction Amount],[Net Payable Amount] from
                        $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                        $getDetail = DB::select(DB::raw($query));
                        $deduction=$getDetail[0]->{$columnName};
                        $TotalDeductionAmount=$getDetail[0]->{'Total Deduction Amount'};
                        $TotalDeductionAmount=($TotalDeductionAmount-$deduction) + $updateValue;
                        $NetPayableAmount=$getDetail[0]->{'Net Payable Amount'};
                        $NetPayableAfterDeduction=$NetPayableAmount - ($TotalDeductionAmount);

                        DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
                            'updatefields' => "[$columnName]='$updateValue',[Total Deduction Amount]='$TotalDeductionAmount',[Net Payable After Deduction]='$NetPayableAfterDeduction'",
                                'where' => "where  [Exam Code]='$examType' and [Reference No_] ='$referenceNo' ",
                                'table' => "$TABillHeaderTbl",
                            ]);
                    }


                    }



            // end update



                $query = "select * from $CityCoordinatorOfflineTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                $data = DB::select(DB::raw($query));
                $viewLoad = 'ccfields';


                if (empty($data)) {
                    $query = "select * from $CentreBillTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                    $data = DB::select(DB::raw($query));
                    $viewLoad = 'csfields';
                }

                if (empty($data)) {
                    $query = "select * from $TABillHeaderTbl where [Reference No_]='$referenceNo' and [Observer ID]='$observerId'";
                    $data = DB::select(DB::raw($query));
                    $viewLoad = 'observerfields';

                    $query = "select * from $TABillLocalTbl where [Reference No_]='$referenceNo'";
                    $TABillLocalData = DB::select(DB::raw($query));
                    $query = "select * from $TABillHotelTbl where [Reference No_]='$referenceNo'";
                    $TABillHotelData = DB::select(DB::raw($query));
                    $query = "select * from $TABillOutstationTbl where [Reference No_]='$referenceNo'";
                    $TABillOutstationData = DB::select(DB::raw($query));
                }


            // If no data was found from any of the tables
            if (empty($data)) {
                return response()->json(['error' => 'No records found for the provided reference number and observer ID.'], 404);
            }

            $emode = "select Exam_Mode from $examTable where [Exam Type]='$examType'";
            $exam_mode = DB::select(DB::raw($emode));

            $examMode=$exam_mode[0]->{'Exam_Mode'};

            if($examMode)
            {
                if($examMode=='CBT')
                {
                   $Mode='Online';
                }elseif($examMode=='PNP')
                {
                    $Mode='Offline';
                }
            }
            $query = "select * from $VendorBankTbl where [No_]='$observerId'";
            $VendorBank = DB::select(DB::raw($query));
            $query = "select [P_A_N_ No_] from $VendorPanTbl where [No_]='$observerId'";
            $VendorPan = DB::select(DB::raw($query));
            $query = "select * from $VendorContactTbl where [No_]='$observerId'";
            $VendorContact = DB::select(DB::raw($query));
            $query = "select [Observer Type] from $ExamProfile where [ObserverID]='$observerId' and [ExamType]='$examType'";
            $ExamProfile = DB::select(DB::raw($query));

            $VendorBank=$VendorBank[0]??[];
            $VendorPan=$VendorPan[0]??[];
            $VendorContact=$VendorContact[0]??[];
            $data=$data[0]??[];
            $role=config('constants.RoleName.'.$ExamProfile[0]->{'Observer Type'});
            $referenceNo=$data->{'Reference No_'};

            $htmlView = view('admin.'.$viewLoad, compact('data', 'VendorBank', 'VendorPan', 'VendorContact', 'role', 'examType', 'billTable','Billstatus','TABillLocalData', 'TABillHotelData', 'TABillOutstationData', 'Mode', 'referenceNo'))->render();

            // Return JSON with both the view and additional data
            return response()->json([
                'html' => $htmlView,  // This contains the full view
                'role' => $role,
                'Mode' => $Mode,
                'updateStatus'=>$success,
                'referenceNo' => $referenceNo,
            ]);

        } catch (\Exception $e) {
            // Handle any errors
            return response()->json(['error' => 'Query execution failed: ' . $e->getMessage()], 500);
        }
    }

    public function generatePdf($referenceNo = null, $examType = null , $observerID = null, $isNew = 1, $Role=null)
    {
        $anotherController = app()->make(OldReportController::class);
        if($isNew == 0){
            if($Role == 14){
                return $anotherController->csOldReport($referenceNo, $examType, $observerID, 'pdf');
            }elseif($Role == 1){
                return $anotherController->ccOldReport($referenceNo, $examType, $observerID, 'pdf');
            }
        }
    }


    public function generateBulkPdf(Request $request){
        $anotherController = app()->make(OldReportController::class);
        $anotherNewController = app()->make(PdfDownloadController::class);
        foreach($_POST['selectedBills'] as $key => $val){
            $vals  =  explode("=", $val);

            $referenceNo[] = $vals[0];
            $examType[] = $vals[4];
            $observerID[] = $vals[3];
            $role[] =  $vals[1];
            $isNewBillType = $vals[2];
            $centers[] = $vals[5];
        }

        if($isNewBillType[0] == 0){
            if($role[0] == 14){
                return $anotherController->csOldReport($referenceNo,$examType ,$observerID, 'pdf');
            }elseif($role[0] == 1){
                return $anotherController->ccOldReport($referenceNo,$examType ,$observerID, 'pdf');
            }else{

            }
        }else{
            if($role[0] == 14){
                return $anotherNewController->CS_PDF_Report($referenceNo,$examType ,$role, 'pdf',$observerID,$centers);
            }elseif($role[0] == 1){
                return $anotherNewController->downloadPdf($referenceNo,$examType ,$role, 'pdf',$observerID,$centers);
            }else{
                return $anotherNewController->OBS_PDF_Report($referenceNo,$examType,$role, 'pdf',$observerID,$centers);
            }
        }
    }



}
