<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Auth;
use App\Services\MasterServices;

use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Str;

class AdminLoginController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        $title = 'Login';
        return view('admin.auth.index', compact('title'));
    }


    public function adminlogin(Request $request)
    {

        if ($request->isMethod('post')) {
            $validator = Validator::make($request->all(), ['password' => 'required', 'username' => 'required']);
            if ($validator->fails()) {
                return redirect()->back()->withErrors($validator);
            }
            $credentials = [
                'username' => $request->username,
                'password' => $request->password
            ];
            
            if (Auth::attempt($credentials)) {

                $user = Auth::user();

                if (in_array($user->Role_ID, [24,25,26, 27])) {
                    // Log successful login
                    MasterServices::customlog('login', 'Admin login successful');
                   // Session::flash('success', 'Login successful!');

                    if (in_array($user->Role_ID, [24,25,26])) {
                        //return redirect()->intended('/admin/dashboard');
                        return redirect()->route('admin.dashboard');
                    } elseif ($user->Role_ID == 27) {
                        //return redirect()->intended('/admin/superdashboard');
                        return redirect()->route('admin.superdashboard');
                    }
                } else {
                    // Logout and redirect if the user is not authorized
                    Auth::logout();
                    return back()->withErrors([
                        'role' => 'You are not authorized to access this area.',
                    ]);
                }

                // MasterServices::customlog('login', 'login user');
                // Session::flash('success', 'Login successful!');
                // return redirect()->intended('/admin/dashboard');
            }

            return back()->withErrors([
                'username' => 'The provided credentials do not match our records.',
            ]);
        } elseif (Session::has('guest_data')) {
            return view('auth.login');
        } else {
            return redirect('admin.index');
        }
    }

    public function forgotPassword(Request $request){
        return view('admin.auth.forgot-password');
    }

    public function forgotPasswordPost(Request $request){
        try {
            $validator = Validator::make($request->all(), [
                'username' => 'required|max:20',
                'email' => ['required', 'email', 'regex:/^[\w\.]+@[a-zA-Z0-9\-]+\.[a-zA-Z]{2,6}$/'],
            ]);

            if ($validator->fails()) {
                return redirect()->back()->withErrors($validator);
            }

            $username = $request->username;
            $adminemail = $request->email;

            $user = User::where('UserName', $username)->where('Email',$adminemail)->first();
            if (!$user) {
                return redirect()->route('admin.admin-forgot-password-link')->with('userError', 'User Not Found');
            }else{
                $Name = $user->FullName ?? 'User';
                $userPassword = $user->PasswordHash;
                $token = Str::random(60);
                $token_expires_at = now()->addMinutes(10)->timestamp;
                $finalTokenNew = $token.'_'.$token_expires_at;
            
                $password = Crypt::encrypt($userPassword);
                $finalToken = $password.'_'.$finalTokenNew;
                $resetLink = url('/admin/reset-passwords/' . $finalToken);

                if(!empty($username)){
                    DB::table('FRONT_ERP_USER_AdminUser')->where('UserName',$username)->where('Email',$adminemail)->update(['PasswordResetToken'=>$finalToken]);
                    $To = $adminemail;
                    $TEMPLATE_UID = '66bb247747867';
                    $PARAMS = array('CANDIDATENAME'=>ucfirst($Name),'PROJECTNAME'=>'Team National Testing Agency','LINK'=>$resetLink);
                    $sentEmail = resetPasswordMail($To,$TEMPLATE_UID,$PARAMS);

                    if($sentEmail){
                        return redirect()->route('admin.login')->with('success', 'A password reset link has been sent to your email. Please check your inbox');
                    }else{
                        return back()->withErrors('Mail not end');
                    }
                }else{
                    return back()->withErrors('Something went to wrong');
                }
            }
        } catch (\Exception $e) {
            Log::error('Error while processing token: ' . $e->getMessage());
            return redirect()->route('admin.login')->withErrors('An error occurred. Please try again later.');
        }
    }

    public function showResetForm(Request $request,$token){
        try {
            $stringUrl = explode('_',$token);
            $tokenPass = $stringUrl[0];
            $expiryTime = $stringUrl[2];
            $OldDecryptedpassword = Crypt::decrypt($tokenPass);
            $countToken = DB::table('FRONT_ERP_USER_AdminUser')->where('PasswordHash',$OldDecryptedpassword)->where('PasswordResetToken',$token)->count();
            if($countToken>0){
                if (now()->timestamp > $expiryTime) {
                    return redirect()->route('admin.login')->withErrors('The token has expired!');
                } else {
                    return view( 'admin.auth.set_reset_password',compact('token'));
                }
            }else{
                return redirect()->route('admin.login')->withErrors('The link not valid');
            }
        } catch (\Exception $e) {
            Log::error('Error while processing token: ' . $e->getMessage());
            return redirect()->route('admin.login')->withErrors('An error occurred. Please try again later.');
        }
    }

    public function setForgotPassword(Request $request){
        $request->validate([
            'password'=>'required',
            'confirmpassword' => 'required|same:password',
        ]);
        $newPassword = trim($request->input('password'));
        $tokenPass = $request->input('token'); 

        $changePassword = Hash::make($newPassword);
        $updatePass = DB::statement('exec updateCommonDataSP :updatefields, :where, :table', [
            'updatefields' => "PasswordHash='$changePassword'",
            'where' => "where PasswordResetToken='$tokenPass'",
            'table' => "FRONT_ERP_USER_AdminUser",
        ]);
        if($updatePass){
            return redirect()->route('admin.login')->with('success', 'Your password has been changed successfully.');
        }else{
            return back()->withErrors('something went to wrong');
        } 
    }

}