<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\DB;
class BillReportController extends Controller
{
    public function index (Request $request)
    {
        $CityCoordinatorOfflineTbl = config('constants.CityCoordinatorOfflineTbl');
        $TABillHeaderTbl = config('constants.TABillHeaderTbl');
        $CentreBillTbl = config('constants.CentreBillTbl');
        $ExamProfile = config('constants.ExamProfile');

        // Escape table names for SQL Server
        $CityCoordinatorOfflineTbl = '[' . str_replace(['[', ']'], '', $CityCoordinatorOfflineTbl) . ']';
        $CentreBillTbl = '[' . str_replace(['[', ']'], '', $CentreBillTbl) . ']';
        $TABillHeaderTbl = '[' . str_replace(['[', ']'], '', $TABillHeaderTbl) . ']';
        $ExamProfile = '[' . str_replace(['[', ']'], '', $ExamProfile) . ']';


        $perPage = $request->perPage ?? 15; // Set perPage directly

        // Assign search filters
        $referenceNo = trim($request->referenceNo) ?? null;
        $ntaId = trim($request->ntaId) ?? null;
        $examType = trim($request->examType) ?? null;
        $userType = trim($request->userType) ?? null;

        $isSearch = !empty($referenceNo) || !empty($ntaId) || !empty($examType) || !empty($userType);
        if ($isSearch) {
            if(!empty($userType))
            {
                if($userType=='CC'){
                    $query = DB::table(DB::raw($CityCoordinatorOfflineTbl));           
                    $query->select([
                        DB::raw("[Reference No_] AS referenceNo"),
                        DB::raw("[Observer ID] AS observerId"),
                        DB::raw("[Exam Type] AS examType"),
                        DB::raw("[Advance Payment] AS advancePayment"),
                        DB::raw("[Net Payable After Deduction] AS netPayable"),
                        DB::raw("[Bill Status] AS billStatus")
                    ]);  
                    if (!empty($examType)) {
                        $query->where('Exam Type', $examType);
                        }                              
                }elseif($userType=='CS')
                {
                    $query = DB::table(DB::raw($CentreBillTbl));           
                    $query->select([
                        DB::raw("[Reference No_] AS referenceNo"),
                        DB::raw("[Observer ID] AS observerId"),
                        DB::raw("[Exam Type] AS examType"),
                        DB::raw(" '00.00' AS advancePayment"),
                        DB::raw("[Net Payable After Deductions] AS netPayable"),
                        DB::raw("[Bill Status] AS billStatus")
                    ]); 
                    if (!empty($examType)) {
                        $query->where('Exam Type', $examType);
                        }                
                }elseif($userType=='OB')
                {
                    $query = DB::table(DB::raw($TABillHeaderTbl));    
                    $query->select([
                        DB::raw("[Reference No_] AS referenceNo"),
                        DB::raw("[Observer ID] AS observerId"),
                        DB::raw("[Exam Code] AS examType"),
                        DB::raw("[Advance Payment] AS advancePayment"),
                        DB::raw("[Net Payable After Deduction] AS netPayable"),
                        DB::raw("[Bill Status] AS billStatus")
                    ]); 
                    if (!empty($examType)) {
                        $query->where('Exam Code', $examType);
                        }  
                }  
                if (!empty($referenceNo)) {
                    $query->where('Reference No_', $referenceNo);
                }
                if (!empty($ntaId)) {
                    $query->where('observer id', $ntaId);
                }
    
                $BillReport = $query->paginate($perPage);
            }else{
                $queries = [];
                $i = 0;
                $tables = [$TABillHeaderTbl, $CityCoordinatorOfflineTbl, $CentreBillTbl];
                // echo "$tables[0]";
                // die;  
                foreach ($tables as $table) {
                    $query = DB::table(DB::raw("$table"));
                
                    // Select statements
                    if ($i == 0) {
                        $query->select([
                            DB::raw("[Reference No_] AS referenceNo"),
                            DB::raw("[Observer ID] AS observerId"),
                            DB::raw("[Exam Code] AS examType"),
                            DB::raw("[Advance Payment] AS advancePayment"),
                            DB::raw("[Net Payable After Deduction] AS netPayable"),
                            DB::raw("[Bill Status] AS billStatus")
                        ]);
                    }elseif($i==2)
                    {
                        $query->select([
                            DB::raw("[Reference No_] AS referenceNo"),
                            DB::raw("[Observer ID] AS observerId"),
                            DB::raw("[Exam Type] AS examType"),
                            DB::raw(" '00.00' AS advancePayment"),
                            DB::raw("[Net Payable After Deductions] AS netPayable"),
                            DB::raw("[Bill Status] AS billStatus")
                        ]);                        
                    }
                    else {
                        $query->select([
                            DB::raw("[Reference No_] AS referenceNo"),
                            DB::raw("[Observer ID] AS observerId"),
                            DB::raw("[Exam Type] AS examType"),
                            DB::raw("[Advance Payment] AS advancePayment"),
                            DB::raw("[Net Payable After Deduction] AS netPayable"),
                            DB::raw("[Bill Status] AS billStatus")
                        ]);
                    }
                
                    // Apply filters if they exist
                    if (!empty($referenceNo)) {
                        $query->where(DB::raw("[Reference No_]"), '=', $referenceNo);
                    }
                    if (!empty($ntaId)) {
                        $query->where(DB::raw("[Observer ID]"), '=', $ntaId);
                    }
                    if (!empty($examType)) {
                        $query->where(DB::raw($i == 0 ? "[Exam Code]" : "[Exam Type]"), '=', $examType);
                    }
                
                    $queries[] = $query;
                    $i++;
                }
                
                // Merge results manually
                $mergedResults = collect();
                foreach ($queries as $query) {
                    $mergedResults = $mergedResults->merge($query->get());
                }
                
                $BillReport = $this->paginateCollection($mergedResults, $perPage);
            }
        }else{
            $BillReport = new LengthAwarePaginator(new Collection(), 0, $perPage);
        }

        $title = 'Bill Report';

        return view('admin.bill_report.index', compact('BillReport','title'));
    }
    private function paginateCollection($items, $perPage)
    {
        $page = Paginator::resolveCurrentPage() ?: 1;
        $items = $items instanceof Collection ? $items : Collection::make($items);

        return new LengthAwarePaginator(
            $items->forPage($page, $perPage),
            $items->count(),
            $perPage,
            $page,
            ['path' => request()->url(), 'query' => request()->query()]
        );
    }
    
}
