<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Session;
use App\Http\Requests\AddNewExamRequest;
use Illuminate\Support\Facades\Validator;
use JsValidator;
use App\Imports\ExamImport;
use DateTime;
use Exception;
use Maatwebsite\Excel\Facades\Excel;

class ExamMasterController extends Controller
{


    public function index(Request $request)
    {

        //$total_per_page_data =  config('constants.total_per_page_data');
        $per_page_count = config('constants.per_page_count');
        ksort($per_page_count);
        $total_per_page_data = array_key_first($per_page_count);
        //$firstValue = $per_page_count[$total_per_page_data];
        if (isset($request->perPage) && $request->perPage) {
            $total_per_page_data = $request->perPage;
        }
        //prd($per_page_count);
        // Handle search query
        $searchQuery = $request->input('search'); // Get the search term from the request
        $title = "Exam Master List";
        $examList = getAllExamsList($total_per_page_data, $searchQuery);

        return view('admin.examMaster.examsList', ['examList' => $examList, 'per_page_count' => $per_page_count, 'title' => $title]);
    }
    public function add()
    {
        $Exam_Mode = config('constants.Exam_Mode');
        $ExamShift = config('constants.ExamShift');
        $Is_ExamOver = config('constants.Is_ExamOver');
        $validator = JsValidator::make((new AddNewExamRequest())->rules(), (new AddNewExamRequest())->messages(), (new AddNewExamRequest())->attributes());
        $title = "Add New Exam";
        return view('admin.examMaster.addexams', ['Exam_Mode' => $Exam_Mode, 'Is_ExamOver' => $Is_ExamOver, 'ExamShift' => $ExamShift, 'validator' => $validator, 'title' => $title]);
    }



    public function saveExamDetails(AddNewExamRequest $request)
    {
        try {
            if ($request->isMethod('post')) {

                // Initialize required configurations
                $examDatesTable = config('constants.ExamDates');
                $examShiftTable = config('constants.ExamShiftTable');

                // Retrieve form inputs
                $examName = $request->exam_name;
                $examType = ($request->hiddenExamType) ? $request->hiddenExamType : str_replace(' ', '', $examName) . $request->ExamYear;
                $examDisp = ($request->hiddenExamDisp) ? $request->hiddenExamDisp : $examName . ' ' . $request->ExamYear;
                $examMode = $request->exam_mode;
                $examYear = $request->ExamYear;
                $examStatus = $request->exam_status;
                $examDate = $request->ExamDate;
                $examMaster = $request->ExamMaster;

                $allDates = [];

                // Collect all date keys from fromOne and toOne arrays
                foreach (["fromOne", "toOne"] as $key) {
                    foreach ($examMaster[$key] as $subArray) {
                        $allDates = array_merge($allDates, array_keys($subArray));
                    }
                }
                $minDate = min($allDates);
                $maxDate = max($allDates);

                $ExamFromDate = ($minDate) ? date('d-m-Y',strtotime($minDate)) : '';
                $ExamToDate = ($maxDate) ? date('d-m-Y',strtotime($maxDate)) : '';
                $ExamDuration = $ExamFromDate.' To '.$ExamToDate;
                $beforeMinDate = (new DateTime($minDate))->modify('-1 day')->format('Y-m-d');
                $examMockDate = ($beforeMinDate) ? date('Y-m-d',strtotime($beforeMinDate)) : '';

                $fromOne = $examMaster['fromOne'];
                $toOne = $examMaster['toOne'];


                // Initialize result array
                $result = [];
                

                if(isset($request->daysMock) && $request->daysMock == 1 ){
                    $mockval = 1;
                }
                else{
                    $mockval = 0;
                }
                // Get the max Exam Type Id to determine the next ID
                // $maxExamID = DB::table($examDatesTable)
                //     ->selectRaw('MAX([Exam Type Id]) as examID')
                //     ->first();
                // $newExamID = $maxExamID && isset($maxExamID->examID) ? ($maxExamID->examID + 1) : 1;

                // Prepare result array with shifts and times
                foreach ($examDate as $key => $value) {
                    foreach ($fromOne as $shiftIndex => $shiftArray) {
                        foreach ($shiftArray as $date => $shiftValue) {

                            if (!isset($result[$date])) {
                                $result[$date] = [];
                            }

                            if (isset($fromOne[$shiftIndex][$date]) && isset($toOne[$shiftIndex][$date])) {
                                $fromTime = $fromOne[$shiftIndex][$date];
                                $toTime = $toOne[$shiftIndex][$date];
                                $result[$date][$fromTime . '-' . $toTime] = $fromTime . '-' . $toTime;
                            }
                        }
                    }
                }

                // // Sort shifts for each date
                foreach ($result as &$times) {
                    uksort($times, function ($a, $b) {
                        return strtotime(explode('-', $a)[0]) - strtotime(explode('-', $b)[0]);
                    });
                }


               DB::beginTransaction();

                // Check if updating or inserting new record
                if (!empty($request->hiddenExamID)) {
                    DB::table($examDatesTable)
                        ->where('Exam Type Id', $request->hiddenExamID)
                        ->update([
                            //'Exam Type' => $examType,
                            'Exam Name' => $examName,
                            'Exam_Mode' => $examMode,
                            'Year' => $examYear,
                            'ExamDisp' => $examDisp,
                            'Is_Active' => $examStatus,
                            'Exam_Duration' =>  $ExamDuration,
                            'Mock_Date' => $examMockDate,
                            'mockval'=> $mockval
                        ]);
                    $lastInsertId = $request->hiddenExamID;
                } else {
                    // DB::table($examDatesTable)->insert([
                    //     'Exam Type Id' => $newExamID,
                    //     'Exam Type' => $examType,
                    //     'Exam Name' => $examName,
                    //     'Exam_Mode' => $examMode,
                    //     'Year' => $examYear,
                    //     'ExamDisp' => $examDisp,
                    //     'Is_Active' => $examStatus,
                    //     'Is_Active' => 1,
                    //     'Exam_Duration' =>  $ExamDuration,
                    //     'Mock_Date' => $examMockDate,
                    //     'mockval'=> $mockval
                    // ]);
                    // $lastInsertId = $newExamID;
                    
                    $lastInsertId = DB::table($examDatesTable)->insertGetId([
                        'Exam Type' => $examType,
                        'Exam Name' => $examName,
                        'Exam_Mode' => $examMode,
                        'Year' => $examYear,
                        'ExamDisp' => $examDisp,
                        'Is_Active' => $examStatus,
                        'Exam_Duration' =>  $ExamDuration,
                        'Mock_Date' => $examMockDate,
                        'mockval' => $mockval
                    ]);
                    
                    // Now you can use the $lastInsertId variable, which contains the ID of the last inserted row.
                    
               
                }

                // Build an array to set all Date_1 to Date_12 to NULL
                $datesToNull = [];
                for ($i = 1; $i <= 12; $i++) {
                    $datesToNull['Date_' . $i] = null;
                }

                // Set all Date_1 to Date_12 columns to NULL
                DB::table($examDatesTable)
                    ->where('Exam Type Id', $lastInsertId)
                    ->update($datesToNull);

                // Update the exam dates table with the exam dates
                $examDateIndex = 0;
                foreach ($result as $index => $examShifts) {
                    $examDateIndex++;
                    DB::table($examDatesTable)
                        ->where('Exam Type Id', $lastInsertId)
                        ->update(['Date_' . $examDateIndex => $index]);
                }

                $nullFields = [];
                for ($dIndex = 1; $dIndex <= 12; $dIndex++) {
                    for ($sIndex = 1; $sIndex <= 4; $sIndex++) {
                        $field = "D{$dIndex}S{$sIndex}";
                        $nullFields[$field] = '';
                    }
                }

                $examTypeCount = DB::table($examShiftTable)
                    ->where('Exam Type', $request->hiddenExamType)
                    ->count();
                //echo $examTypeCount;die;

                // Set all D{dateIndex}S{shiftIndex} fields to NULL if the count is greater than 0
                if ($examTypeCount > 0) {
                    DB::table($examShiftTable)
                        ->where('Exam Type', $request->hiddenExamType)
                        ->update($nullFields);

                    $ExamShiftType = $request->hiddenExamType;
                } else {
                    // Insert a new record if no matching records are found
                    $lastInsertIdShift = DB::table($examShiftTable)->insertGetId([
                        'Exam Type' => $examType
                    ]);
                    $ExamShiftType = $examType;
                }

                // Prepare update string for stored procedure
                $updateString = '';
                $dateIndex = 0;
                foreach ($result as $dateKey => $shiftArray) {
                    $dateIndex++;
                    $shiftIndex = 0;
                    foreach ($shiftArray as $shiftValue) {
                        $shiftIndex++;
                        $updateString .= "D{$dateIndex}S{$shiftIndex}='" . $shiftValue . "',";
                    }
                }
                $updateString = rtrim($updateString, ',');

                // Update Exam Shift Table using Stored Procedure
                DB::update("exec updateCommonDataSP :updatefields, :where, :table", [
                    'updatefields' => $updateString,
                    'where' => "Where [Exam Type]='$ExamShiftType'",
                    'table' => "[$examShiftTable]"
                ]);

                DB::commit();
                return redirect()->route('exam.index')->with('success', 'Exam added successfully!');
            }
        } catch (\Exception $e) {
            DB::rollBack();
            $message = $e->getMessage();
            return redirect()->back()->with('error123', $message);
        }
    }


    public function edit(Request $request)
    {
        $exam_id = decrypt(urldecode($request->get('exam_id')), config('constants.encryptionKey'));

        // Configuration variables
        $config = config('constants');
        $ExamDates = $config['ExamDates'];
        $ExamShiftTable = $config['ExamShiftTable'];
        $ExamShift = $config['ExamShift'];
        $Exam_Mode = $config['Exam_Mode'];
        $Is_ExamOver = $config['Is_ExamOver'];
        $labelName = "Update Exam Details";
        $title = "Update Exam";


        $examData = DB::selectOne("exec getCommonDataSP :select, :input, :table", [
            'select' => "ED.*",
            'input' => "Left join [$ExamShiftTable] ES on ES.[Exam Type] = ED.[Exam Type] WHERE ED.[Exam Type Id]=$exam_id",
            'table' => "[$ExamDates] ED"
        ]);

        $examDuration = $examData->Exam_Duration;
        $examDurationArr = explode('To', $examDuration);
        //prd($examDurationArr);

        $transformedArray = [
            'DateType' => [],
            'ExamDetails' => []
        ];

        // Extract Exam Dates
        for ($i = 1; $i <= 12; $i++) {
            $dateKey = "Date_$i";
            if (!empty($examData->{$dateKey}) && $examData->{$dateKey} != '1753-01-01 00:00:00.000') {
                $transformedArray['DateType'][] = $dateKey;
                $transformedArray['ExamDetails'][] = [
                    'ExamDate' => date('Y-m-d', strtotime($examData->{$dateKey})),
                    'Examshift' => []
                ];
            }
        }

        // Get Exam Type
        $examType = $examData->{'Exam Type'};

        // Retrieve shifts for this exam type
        $selectedExamShifts = DB::selectOne("exec getCommonDataSP :select, :input, :table", [
            'select' => "*",
            'input' => "WHERE [Exam Type]='$examType'",
            'table' => "[$ExamShiftTable] ES"
        ]);

        $examShiftVal = [];

        if (!empty($selectedExamShifts)) {
            foreach ($selectedExamShifts as $key => $value) {
                if (preg_match('/D(\d+)S(\d+)/', $key, $matches) && !empty($value)) {
                    $day = (int) $matches[1] - 1;
                    $shift = (int) $matches[2] - 1;
                    $examShiftVal[$day][$shift] = $value;
                }
            }
        }



        // Link shifts to ExamDate in $transformedArray
        foreach ($transformedArray['ExamDetails'] as $index => &$examDate) {
            $dateIndex = $index;
            if (isset($examShiftVal[$dateIndex])) {
                foreach ($examShiftVal[$dateIndex] as $shiftTime) {
                    list($from, $to) = explode('-', $shiftTime);
                    $examDate['Examshift'][] = [
                        'fromOne' => $from,
                        'ToOne'   => $to
                    ];
                }
                sort($examDate['Examshift']);
            }
        }

       // prd($transformedArray);

        // Return View
        return view('admin.examMaster.addexams', [
            'Exam_Mode' => $Exam_Mode,
            'Is_ExamOver' => $Is_ExamOver,
            'examData' => $examData,
            'labelName' => $labelName,
            'transformedArray' => $transformedArray,
            'ExamShift' => $ExamShift,
            'selectedExamShifts' => (array)$examShiftVal,
            'title' => $title,
            'examDurationArr' => $examDurationArr
        ]);
    }

    public function import(Request $request)
    {


        try {

            // Allowed MIME types for the file upload
            $allowedMimes = [
                'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
                'application/vnd.ms-excel',
                'text/csv'
            ];

            // Get the uploaded file
            $file = $request->file('importFile');

            // Check if file is uploaded
            if (!$file) {
                return redirect()->route('exam.index')->with('warning', 'No file uploaded. Please upload an Excel file.');
            }
            // Get file MIME type
            $mimeType = $file->getMimeType();

            // Check if the file's MIME type is allowed
            if (!in_array($mimeType, $allowedMimes)) {
                return redirect()->route('exam.index')->with('warning', 'Invalid file format. Please upload a valid Excel file (xlsx, xls, or csv).');
            }

            // Check file size (2MB limit)
            $maxFileSize = 2 * 1024 * 1024; // 2MB in bytes
            if ($file->getSize() > $maxFileSize) {
                return redirect()->route('exam.index')->with('warning', 'File size should not exceed 2MB.');
            }


            // Import the file using the ExamImport class
            Excel::import(new ExamImport, $file);
            return redirect()->route('exam.index')->with('success', 'Data imported successfully!');
        } catch (\Exception $ex) {
            $errorMessage = str_replace(['&quot;', '"'], '', html_entity_decode($ex->getMessage(), ENT_QUOTES));
            return redirect()->route('exam.index')->with('error', 'Something went wrong! Please check if the Excel is empty or compare it with the sample or Excel is larger than 2MB or ' . $errorMessage);
        }
    }

    public function checkExamTypeExist(Request $request)
    {
        $ExamName = $request->input('ExamName');
        $ExamYear = $request->input('ExamYear');
        $examDatesTable = config('constants.ExamDates');
        // Your logic to check if the exam type exists in the database
        $exists = DB::table($examDatesTable)->where('Exam Name', $ExamName)->where('Year', $ExamYear)->exists();

        if ($exists) {
            return response()->json([
                'status' => true,
                'message' => 'Exam Name and Year already exists.',
            ]);
        }

        return response()->json([
            'status' => false,
            'message' => 'Exam Name and Year is available.',
        ]);
    }

    public function updateExamStatus(Request $request)
    {
        $examTypeId = $request->input('examTypeId');
        $isActive = $request->input('isActive');
        $examDatesTable = config('constants.ExamDates');
        $result = DB::table($examDatesTable)
            ->where('Exam Type Id', $examTypeId)
            ->update(['Is_Active' => $isActive]);

        return response()->json([
            'success' => $result !== 0,
            'message' => $result !== 0 ? 'Status has been changed successfully.' : 'Error updating status'
        ]);
    }
}
