<?php

namespace App\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;

class FinancialNormsRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array<string, mixed>
     */
    public function rules()
    {
        return [
            'source_type' => 'required',
            'source_code' => 'required',
            'source_name' => 'required|string|max:255',
            // 'remuneratlon' => 'required',
            'remuneratlon_amount_s1' => 'required|integer',
            'remuneratlon_amount_s2' => 'required|integer',
            'remuneratlon_amount_s3' => 'required|integer',
            'remuneratlon_amount_s4' => 'required|integer',
            'mock_drilling_amount' => 'required|integer',
            'mock_drilling_conveyance' => 'required|integer',
            'candidates' => 'required|string',
            'exam_type' => 'required|string',
            'exam_type_id' => 'required', 
            'exam_mode' => 'required',
            'exam_name' => 'required|string',
            'conveyance_amount' =>  'required|integer',
            'food_amount' => 'required|integer',
            'no_of_person' => 'required|integer',
            'no_of_days' => 'required|integer',
            'fixed_amount' => 'required|integer',
            // 'mock_drill_text' => 'required|string',
            'attechment' => 'required',
            // 'attechment_required' => 'required',
            'read_only' => 'required',
            'required' => 'required',
            'display_order' => 'required',
            'tds' => 'required',
            // 'tdsprecentage' => 'required',
            // 'field_name' => 'required',
            // 'mock_drill_filed_name' => 'required',
            'is_active' => 'required',
            'is_extra_field' => 'required',
        ];
    }
    

    public function messages()
    {
        return [
            'source_type.required' => 'Role cannot be blank.',
            'source_code.required' => 'Category cannot be blank.',
            'source_name.required' => 'Category Text cannot be blank.',
            'norms.required' => ':attribute cannot be blank.',
            'norms.string' => ':attribute must be a string.',
            'norms.min' => ':attribute should  be at least 2 characters.',
            'norms.max' => ':attribute must not exceed 255 characters.',

            // 'remuneratlon.required' => ':attribute cannot be blank.',
            // 'remuneratlon.string' => ':attribute must be a string.',
            // 'remuneratlon.min' => ':attribute should  be at least 2 characters.',
            // 'remuneratlon.max' => ':attribute must not exceed 255 characters.',

            'remuneratlon_amount_s1.required' => ':attribute cannot be blank.',
            'remuneratlon_amount_s1.integer' => ':attribute must be a integer.',

            'remuneratlon_amount_s2.required' => ':attribute cannot be blank.',
            'remuneratlon_amount_s2.integer' => ':attribute must be a integer.',

            'remuneratlon_amount_s3.required' => ':attribute cannot be blank.',
            'remuneratlon_amount_s3.integer' => ':attribute must be a integer.',

            'remuneratlon_amount_s4.required' => ':attribute cannot be blank.',
            'remuneratlon_amount_s4.integer' => ':attribute must be a integer.',
            
            'mock_drilling_amount.required' => ':attribute cannot be blank.',
            'mock_drilling_amount.integer' => ':attribute must be a integer.',
            
            'mock_drilling_conveyance.required' => ':attribute cannot be blank.',
            'mock_drilling_conveyance.integer' => ':attribute must be a integer.',

            'candidates.required' => ':attribute cannot be blank.',
            'candidates.string' => ':attribute must be a string.',
            'candidates.min' => ':attribute should  be at least 2 characters.',
            'candidates.max' => ':attribute must not exceed 255 characters.',

            'exam_type.required' => 'Exam Name is required.',
            'exam_type.string' => ':attribute must be a string.',
            'exam_type.max' => ':attribute must not exceed 15 characters.',
            
            'exam_mode.required' => ':attribute is required.',
            'exam_mode.string' => ':attribute must be a string.',
            'exam_mode.in' => ':attribute must be either CBT or PNP.',

            'exam_name.required' => ':attribute cannot be blank.',
            'exam_name.string' => ':attribute must be a string.',
            'exam_name.min' => ':attribute should  be at least 2 characters.',
            'exam_name.max' => ':attribute must not exceed 255 characters.',

            'conveyance_amount.required' => ':attribute cannot be blank.',
            'conveyance_amount.integer' => ':attribute must be a integer.',
            'conveyance_amount.min' => ':attribute should  be at least 2 characters.',
            'conveyance_amount.max' => ':attribute must not exceed 5 characters.',

        ];
    }

    
    public function attributes()
    {
        return [
            'source_type' => 'Source Type',
            'source_code' => 'Source Code',
            // 'norms' => 'Norms',
            'remuneratlon' => 'Remuneratlon',
            'remuneratlon_amount_s1' => 'Remuneratlon Amount S1',
            'remuneratlon_amount_s2' => 'Remuneratlon Amount S2',
            'remuneratlon_amount_s3' => 'Remuneratlon Amount S3',
            'remuneratlon_amount_s4' => 'Remuneratlon Amount S4',
            'mock_drilling_amount' => 'Mock Drilling Amount',
            'mock_drilling_conveyance' => 'Mock Drilling Conveyance',
            'candidates' => 'Candidates',
            'exam_name' => 'Exam Name',
            'exam_type' => 'Exam Type',
            'exam_mode' => 'Exam Mode',
            'source_name' => 'Norms',
        ];
    }


}
