@extends('layouts.adminLayout.dashboardLayout')
@section('styles')
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/sweetalert2@11.7.4/dist/sweetalert2.min.css">
    <style>
        .error {
            color: red;
        }

        .labelName::after {
            content: " *";
            color: red;
            /* You can change the color if needed */
        }
    </style>
@endsection
@section('content')
<!-- Include jQuery library -->

    <main role="main" class="flex-shrink-0">
        <div class="container custom-container">
            <div class="login-5 mt-50 mb-50">
                <div class="container-fluid">
                    <div class="row no-gutters">
                        <div class="col-sm-12">
                            <div class="container-fluid">
                                @if (isset($advanceData) && !empty($advanceData))
                                    {{ Breadcrumbs::render('EditAdvancePayment', $advanceData) }}
                                @else
                                    {{ Breadcrumbs::render('AddAdvancePayment') }}
                                @endif

                                <div class="card">
                                    <div class="card-header d-flex align-items-center justify-content-between py-3 px-3">
                                        <h4 class="mb-0 fw-bold fs-4"><?= $title ?></h4>
                                        <a href="{{ route('list-advance') }}" class="btn btn-danger text-white fs-5"> <i
                                                class="fa fa-angle-double-left" aria-hidden="true"> Back</i></a>
                                    </div>

                                    <div class="card-body">
                                        <div class="custom-form">
                                            {!! Form::open([
                                                'route' => ['update-advance'],
                                                'method' => 'POST',
                                                'enctype' => 'multipart/form-data',
                                                'id' => 'advanceForm',
                                                'autocomplete' => 'off',
                                            ]) !!}
                                            @csrf

                                            <div class="row">
                                            {!! Form::hidden('Advance#No_', old('Advance#No_', $advanceData->{'Advance No_'} ?? ''), []) !!}
                                                <div class="col-md-6">
                                                    <div class="form-group mb-3">
                                                        {!! Form::label('NTA_Observer_ID', 'NTA ID', ['class' => 'text-left form-label fw-bold labelName']) !!}
                                                        <select class="form-control form-control-lg NTA_Observer_ID" name="NTA_Observer_ID" id="NTA_Observer_ID" >
                                                        <option value="{{ isset($advanceData->{'NTA Observer ID'}) ? $advanceData->{'NTA Observer ID'} : '' }}" 
                                                                selected>
                                                            {{ isset($advanceData->{'NTA Observer ID'}) ? $advanceData->{'NTA Observer ID'} : '' }}
                                                        </option>

                                                        </select>
                                                        @error('NTA_Observer_ID')
                                                            <div class="text-danger mb-4">{{ $message }}</div>
                                                        @enderror
                                                    </div>
                                                </div>

                                     
                                                <div class="col-md-6">
                                                    <div class="form-group mb-3">
                                                        {!! Form::label('Exam Type', 'Exam Type', ['class' => 'text-left form-label fw-bold labelName']) !!}
                                                        {!! Form::select('Exam_Type', $examsList, old('Exam_Type', $advanceData->{'Exam Type'} ?? ''), [
                                                            'class' => 'form-control form-select form-select-lg',
                                                            'placeholder' => 'Select Exam Type',
                                                            'disabled' => !empty($advanceData->{'Exam Type'}) ? true : false,
                                                        ]) !!}
                                                        @error('Exam_Type')
                                                            <div class="text-danger mb-4">{{ $message }}</div>
                                                        @enderror
                                                    </div>
                                                </div>

                                                <div class="col-md-6">
                                                    <div class="form-group mb-3">
                                                        {!! Form::label('Observer Name', 'Functionary Name', ['class' => 'text-left form-label fw-bold labelName']) !!}
                                                        {!! Form::text('Observer_Name', old('Observer_Name', $advanceData->{'Observer Name'} ?? ''), [
                                                            'class' => 'form-control form-control-lg userNames',
                                                            'placeholder' => 'Enter Observer Name',
                                                            'onkeypress' => 'return blockSpecialChar(event, "alpha")',
                                                            'minlength' => 3,
                                                        ]) !!}
                                                        @error('Observer_Name')
                                                            <div class="text-danger mb-4">{{ $message }}</div>
                                                        @enderror
                                                    </div>
                                                </div>

                                                <div class="col-md-6">
                                                    <div class="form-group mb-3">
                                                        {!! Form::label('Advance Amount', 'Advance Amount', ['class' => 'text-left form-label fw-bold labelName']) !!}
                                                        {!! Form::text(
                                                            'Advance_Amount',
                                                            old(
                                                                'Advance_Amount',
                                                                isset($advanceData->{'Advance Amount'}) && !empty($advanceData->{'Advance Amount'})
                                                                    ? number_format($advanceData->{'Advance Amount'}, 0, '', '')
                                                                    : '',
                                                            ),
                                                            [
                                                                'class' => 'form-control form-control-lg',
                                                                'placeholder' => 'Enter Advance Amount',
                                                                'onkeypress' => 'return blockSpecialChar(event, "numeric")',
                                                            ],
                                                        ) !!}
                                                        @error('Advance_Amount')
                                                            <div class="text-danger mb-4">{{ $message }}</div>
                                                        @enderror
                                                    </div>
                                                </div>

                                                <div class="form-group text-end">
                                                    {!! Form::submit('Submit', ['class' => 'btn btn-primary text-white mt-2']) !!}
                                                </div>
                                            </div>
                                            {!! Form::close() !!}
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </div>
    </main>
@endsection
@section('script')
    <script type="text/javascript">
        $(document).ready(function() {
            // Validate the form
            $('#advanceForm').validate({
                rules: {

                    'Observer_Name': {
                        required: true,
                        // minlength: 3, // Example: Minimum length of 3 characters
                        // maxlength: 50,
                    },
                    'Exam_Type': {
                        required: true,
                    }
                    ,
                    'NTA_Observer_ID': {
                        required: true,
                        // minlength: 6, // Example: Minimum length of 3 characters
                        // maxlength: 15, // Example: Minimum length of 3 characters
                    }
                    ,
                    'Advance_Amount': {
                        required: true,
                        number: true, // Ensures that the field only accepts numbers
                        min: 1, // Example: Minimum value should be 1
                    }
                },
                messages: {
                    'Observer_Name': {
                        required: 'Please enter the Observer Name',
                        // minlength: 'Observer Name must be at least 3 characters long',
                        // maxlength: 'Observer Name must be last 50 characters long',
                    },
                    'Advance_Amount': {
                        required: 'Please enter the Advance Amount',
                        number: 'Please enter a valid number for Advance Amount',
                        min: 'Advance Amount must be at least 1',
                    },
                    'Exam_Type': {
                        required: 'Please enter the Exam Type',
                    }
                    ,
                    'NTA_Observer_ID': {
                        required: 'Please enter the NTA Observer ID',
                        // minlength: 'NTA Observer ID must be at least 5 characters long',
                        // maxlength: 'NTA Observer ID must be last 9 characters long',
                    }
                },

            });
        });
    </script>


<script>
$(document).ready(function() {
    $('.NTA_Observer_ID').select2();
});    
</script>

<script>

    $(document).ready(function() {
        $('#NTA_Observer_ID').select2({
            placeholder: "Search NTA ID",
            minimumInputLength: 2, // This ensures that the search is triggered only when the input length is at least 2 characters
            ajax: {
                url: '{{ route('ntausers-search') }}', // Laravel route for the AJAX request
                dataType: 'json',
                delay: 250,
                processResults: function (data) {
                    return {
                        results: data.map(function(item) {
                            return {
                                id: item.No_,
                                text: item.No_,
                                namess: item.Name
                            };
                        })
                    };
                },
                cache: true
            }
        });

        //You can also trigger alert on keyup or on select change if needed
        $('#NTA_Observer_ID').on('select2:select', function (e) {
            $(".userNames").val(e.params.data.namess);
        });
    });

</script>

@endsection
